{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# Project 1: Momentum Strategies\n",
    "\n",
    "By: Chengyi (Jeff) Chen"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "### Description of The Project:\n",
    "\n",
    "We want to test the momentum effect on different assets. We have daily price time series information for assets in the following asset classes:\t\t\n",
    "1. Equity Index\t\n",
    "2. Credit (Corporate Bonds)\t\n",
    "3. Govt Bond (Sovereign Bonds)\t\n",
    "4. Currencies\t\n",
    "5. Commodities\t\n",
    "6. Equity Factors\t\n",
    "\n",
    "You choose one of the above 6 asset classes, and we'll build momentum portfolios for each one of the assets in the chosen asset class.\t\t\n",
    "\t\t\n",
    "For every asset, based on its price time series, a momentum portfolio could be built in the following way:\t\t\n",
    "Step 1: starting from 12/31/2001, at each month-end time point, calculate the asset's return in the past 12 months, if the return is positive (negative), the portfolio longs (shorts) 100% of the asset and holds the long (short) position for the next month. \t\n",
    "Step 2:\tcalculate the monthly returns for the above-built portfolio till 2020/8/31.\t\n",
    "Step 3: prepare the deliverable table as follows:\n",
    "\n",
    "<img src=\"./assets/project_1_deliverable.png\" width=\"500px\"/>\n",
    "\n",
    "Bonus problem\n",
    "At step 1, at each month-end date, calculate all asset returns in the past 12 months,  \n",
    "and allocate 50% weight to the top two performing assets respectively; and -50% to the bottom two performing assets respectively in your portfolio.\n",
    "And then do step 2 and step 3."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "application/javascript": [
       "\n",
       "            setTimeout(function() {\n",
       "                var nbb_cell_id = 1;\n",
       "                var nbb_unformatted_code = \"%load_ext nb_black\\n%load_ext autotime\\n\\nimport pandas as pd\\nimport numpy as np\\nfrom typing import Dict\\nfrom datetime import datetime, timedelta\\nfrom abc import ABC, abstractmethod\\n\\n\\ndef get_asset_class_data(raw_data: pd.DataFrame) -> pd.DataFrame:\\n    \\\"\\\"\\\"Splits the Raw dataframe into the separate asset \\n    class dataframes and return them in a MultiIndex Dataframe\\n    \\\"\\\"\\\"\\n    asset_class_df_list = np.split(\\n        raw_data, np.argwhere(raw_data.isnull().all(axis=0).values).flatten(), axis=1\\n    )\\n    asset_class_dfs = {}\\n    for df in asset_class_df_list:\\n        df = df.dropna(how=\\\"all\\\", axis=1)\\n        asset_class_name = df.iloc[0, 0]\\n        columns = df.iloc[0, :].str.cat(df.iloc[1, :], sep=\\\" - \\\")\\n        columns.iloc[0] = \\\"Date\\\"\\n        df.columns = columns\\n        df = df.drop([0, 1], axis=0)\\n        df = df.set_index(\\\"Date\\\")\\n        asset_class_dfs[\\n            asset_class_name\\n        ] = df.sort_index()  # Earliest stock price on top\\n    return pd.concat(asset_class_dfs, names=[\\\"asset_class\\\", \\\"security\\\"], axis=1)\\n\\n\\nclass TradingStrategy(ABC):\\n    def __init__(\\n        self, name: str, asset_class_history: pd.DataFrame, asset_class=\\\"Equity Idx\\\"\\n    ):\\n        \\\"\\\"\\\"\\\"\\\"\\\"\\n        self.name = name\\n        self.asset_class = asset_class\\n        self.asset_class_history = asset_class_history\\n\\n    @abstractmethod\\n    def execute_backtest(self):\\n        \\\"\\\"\\\"\\\"\\\"\\\"\\n        pass\\n\\n    @abstractmethod\\n    def _results(self):\\n        \\\"\\\"\\\"\\\"\\\"\\\"\\n        pass\\n\\n    @staticmethod\\n    def annualized_volatility(returns):\\n        returns = returns.dropna(axis=0)\\n        return np.std(returns) * np.sqrt(len(returns)) if len(returns) > 0 else None\\n\\n    @staticmethod\\n    def annualized_return(returns):\\n        returns = returns.dropna(axis=0)\\n        return (\\n            (\\n                np.prod(np.ones(len(returns)) + np.array(returns)) ** (1 / len(returns))\\n                - 1\\n            )\\n            if len(returns) > 0\\n            else None\\n        )\\n\\n    @staticmethod\\n    def calendar_year_return(returns):\\n        returns = returns.dropna(axis=0)\\n        return (\\n            ((TradingStrategy.annualized_return(returns) + 1) ** 12) - 1\\n            if len(returns) > 0\\n            else None\\n        )\\n\\n    @staticmethod\\n    def sharpe_ratio(returns, R_f=0):\\n        returns = returns.dropna(axis=0)\\n        R_p = TradingStrategy.annualized_return(returns)\\n        \\u03c3_p = TradingStrategy.annualized_volatility(returns)\\n        return (R_p - R_f) / \\u03c3_p if len(returns) > 0 else None\";\n",
       "                var nbb_formatted_code = \"%load_ext nb_black\\n%load_ext autotime\\n\\nimport pandas as pd\\nimport numpy as np\\nfrom typing import Dict\\nfrom datetime import datetime, timedelta\\nfrom abc import ABC, abstractmethod\\n\\n\\ndef get_asset_class_data(raw_data: pd.DataFrame) -> pd.DataFrame:\\n    \\\"\\\"\\\"Splits the Raw dataframe into the separate asset \\n    class dataframes and return them in a MultiIndex Dataframe\\n    \\\"\\\"\\\"\\n    asset_class_df_list = np.split(\\n        raw_data, np.argwhere(raw_data.isnull().all(axis=0).values).flatten(), axis=1\\n    )\\n    asset_class_dfs = {}\\n    for df in asset_class_df_list:\\n        df = df.dropna(how=\\\"all\\\", axis=1)\\n        asset_class_name = df.iloc[0, 0]\\n        columns = df.iloc[0, :].str.cat(df.iloc[1, :], sep=\\\" - \\\")\\n        columns.iloc[0] = \\\"Date\\\"\\n        df.columns = columns\\n        df = df.drop([0, 1], axis=0)\\n        df = df.set_index(\\\"Date\\\")\\n        asset_class_dfs[\\n            asset_class_name\\n        ] = df.sort_index()  # Earliest stock price on top\\n    return pd.concat(asset_class_dfs, names=[\\\"asset_class\\\", \\\"security\\\"], axis=1)\\n\\n\\nclass TradingStrategy(ABC):\\n    def __init__(\\n        self, name: str, asset_class_history: pd.DataFrame, asset_class=\\\"Equity Idx\\\"\\n    ):\\n        \\\"\\\"\\\"\\\"\\\"\\\"\\n        self.name = name\\n        self.asset_class = asset_class\\n        self.asset_class_history = asset_class_history\\n\\n    @abstractmethod\\n    def execute_backtest(self):\\n        \\\"\\\"\\\"\\\"\\\"\\\"\\n        pass\\n\\n    @abstractmethod\\n    def _results(self):\\n        \\\"\\\"\\\"\\\"\\\"\\\"\\n        pass\\n\\n    @staticmethod\\n    def annualized_volatility(returns):\\n        returns = returns.dropna(axis=0)\\n        return np.std(returns) * np.sqrt(len(returns)) if len(returns) > 0 else None\\n\\n    @staticmethod\\n    def annualized_return(returns):\\n        returns = returns.dropna(axis=0)\\n        return (\\n            (\\n                np.prod(np.ones(len(returns)) + np.array(returns)) ** (1 / len(returns))\\n                - 1\\n            )\\n            if len(returns) > 0\\n            else None\\n        )\\n\\n    @staticmethod\\n    def calendar_year_return(returns):\\n        returns = returns.dropna(axis=0)\\n        return (\\n            ((TradingStrategy.annualized_return(returns) + 1) ** 12) - 1\\n            if len(returns) > 0\\n            else None\\n        )\\n\\n    @staticmethod\\n    def sharpe_ratio(returns, R_f=0):\\n        returns = returns.dropna(axis=0)\\n        R_p = TradingStrategy.annualized_return(returns)\\n        \\u03c3_p = TradingStrategy.annualized_volatility(returns)\\n        return (R_p - R_f) / \\u03c3_p if len(returns) > 0 else None\";\n",
       "                var nbb_cells = Jupyter.notebook.get_cells();\n",
       "                for (var i = 0; i < nbb_cells.length; ++i) {\n",
       "                    if (nbb_cells[i].input_prompt_number == nbb_cell_id) {\n",
       "                        if (nbb_cells[i].get_text() == nbb_unformatted_code) {\n",
       "                             nbb_cells[i].set_text(nbb_formatted_code);\n",
       "                        }\n",
       "                        break;\n",
       "                    }\n",
       "                }\n",
       "            }, 500);\n",
       "            "
      ],
      "text/plain": [
       "<IPython.core.display.Javascript object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "%load_ext nb_black\n",
    "%load_ext autotime\n",
    "\n",
    "import pandas as pd\n",
    "import numpy as np\n",
    "from typing import Dict\n",
    "from datetime import datetime, timedelta\n",
    "from abc import ABC, abstractmethod\n",
    "\n",
    "\n",
    "def get_asset_class_data(raw_data: pd.DataFrame) -> pd.DataFrame:\n",
    "    \"\"\"Splits the Raw dataframe into the separate asset \n",
    "    class dataframes and return them in a MultiIndex Dataframe\n",
    "    \"\"\"\n",
    "    asset_class_df_list = np.split(\n",
    "        raw_data, np.argwhere(raw_data.isnull().all(axis=0).values).flatten(), axis=1\n",
    "    )\n",
    "    asset_class_dfs = {}\n",
    "    for df in asset_class_df_list:\n",
    "        df = df.dropna(how=\"all\", axis=1)\n",
    "        asset_class_name = df.iloc[0, 0]\n",
    "        columns = df.iloc[0, :].str.cat(df.iloc[1, :], sep=\" - \")\n",
    "        columns.iloc[0] = \"Date\"\n",
    "        df.columns = columns\n",
    "        df = df.drop([0, 1], axis=0)\n",
    "        df = df.set_index(\"Date\")\n",
    "        asset_class_dfs[\n",
    "            asset_class_name\n",
    "        ] = df.sort_index()  # Earliest stock price on top\n",
    "    return pd.concat(asset_class_dfs, names=[\"asset_class\", \"security\"], axis=1)\n",
    "\n",
    "\n",
    "class TradingStrategy(ABC):\n",
    "    def __init__(\n",
    "        self, name: str, asset_class_history: pd.DataFrame, asset_class=\"Equity Idx\"\n",
    "    ):\n",
    "        self.name = name\n",
    "        self.asset_class = asset_class\n",
    "        self.asset_class_history = asset_class_history\n",
    "\n",
    "    @abstractmethod\n",
    "    def execute_backtest(self):\n",
    "        \"\"\"\"\"\"\n",
    "        pass\n",
    "\n",
    "    @abstractmethod\n",
    "    def _results(self):\n",
    "        \"\"\"\"\"\"\n",
    "        pass\n",
    "\n",
    "    @staticmethod\n",
    "    def annualized_volatility(returns):\n",
    "        returns = returns.dropna(axis=0)\n",
    "        return np.std(returns) * np.sqrt(len(returns)) if len(returns) > 0 else None\n",
    "\n",
    "    @staticmethod\n",
    "    def annualized_return(returns):\n",
    "        returns = returns.dropna(axis=0)\n",
    "        return (\n",
    "            (\n",
    "                np.prod(np.ones(len(returns)) + np.array(returns)) ** (1 / len(returns))\n",
    "                - 1\n",
    "            )\n",
    "            if len(returns) > 0\n",
    "            else None\n",
    "        )\n",
    "\n",
    "    @staticmethod\n",
    "    def calendar_year_return(returns):\n",
    "        returns = returns.dropna(axis=0)\n",
    "        return (\n",
    "            ((TradingStrategy.annualized_return(returns) + 1) ** 12) - 1\n",
    "            if len(returns) > 0\n",
    "            else None\n",
    "        )\n",
    "\n",
    "    @staticmethod\n",
    "    def sharpe_ratio(returns, R_f=0):\n",
    "        returns = returns.dropna(axis=0)\n",
    "        R_p = TradingStrategy.annualized_return(returns)\n",
    "        σ_p = TradingStrategy.annualized_volatility(returns)\n",
    "        return (R_p - R_f) / σ_p if len(returns) > 0 else None"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "application/javascript": [
       "\n",
       "            setTimeout(function() {\n",
       "                var nbb_cell_id = 2;\n",
       "                var nbb_unformatted_code = \"data = get_asset_class_data(\\n    pd.read_excel(\\n        \\\"./data/Project1_TimeSeries_Momentum_Due20200913.xlsx\\\",\\n        sheet_name=2,\\n        header=None,\\n    )\\n)\";\n",
       "                var nbb_formatted_code = \"data = get_asset_class_data(\\n    pd.read_excel(\\n        \\\"./data/Project1_TimeSeries_Momentum_Due20200913.xlsx\\\",\\n        sheet_name=2,\\n        header=None,\\n    )\\n)\";\n",
       "                var nbb_cells = Jupyter.notebook.get_cells();\n",
       "                for (var i = 0; i < nbb_cells.length; ++i) {\n",
       "                    if (nbb_cells[i].input_prompt_number == nbb_cell_id) {\n",
       "                        if (nbb_cells[i].get_text() == nbb_unformatted_code) {\n",
       "                             nbb_cells[i].set_text(nbb_formatted_code);\n",
       "                        }\n",
       "                        break;\n",
       "                    }\n",
       "                }\n",
       "            }, 500);\n",
       "            "
      ],
      "text/plain": [
       "<IPython.core.display.Javascript object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "time: 3.34 s\n"
     ]
    }
   ],
   "source": [
    "data = get_asset_class_data(\n",
    "    pd.read_excel(\n",
    "        \"./data/Project1_TimeSeries_Momentum_Due20200913.xlsx\",\n",
    "        sheet_name=2,\n",
    "        header=None,\n",
    "    )\n",
    ")"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Time Series Momentum Strategy"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "application/javascript": [
       "\n",
       "            setTimeout(function() {\n",
       "                var nbb_cell_id = 3;\n",
       "                var nbb_unformatted_code = \"class TimeSeriesMomentumStrategy(TradingStrategy):\\n    \\\"\\\"\\\"Longs (Shorts) 100% for 1 month if asset's own past 12 month returns > 0 (< 0) \\n    Assumes that all transactions happen on the end of month\\n    \\\"\\\"\\\"\\n\\n    def __init__(\\n        self, momentum_signal_period: int = 12, **kwargs\\n    ):\\n        super().__init__(name=\\\"Time Series Momentum Strategy\\\", **kwargs)\\n        self.momentum_signal_period = momentum_signal_period\\n        self.target_asset_history = self.asset_class_history[\\n            self.asset_class_history.columns[\\n                self.asset_class_history.columns.get_level_values(\\\"asset_class\\\")\\n                == asset_class\\n            ]\\n        ]\\n\\n    def filter_month_end(self, df):\\n        \\\"\\\"\\\"Filter out only data of end of month\\\"\\\"\\\"\\n        dates = df.index\\n        only_month_end_df = df[pd.Series(dates, index=dates).dt.is_month_end]\\n        return only_month_end_df\\n\\n    def check_data(self):\\n        \\\"\\\"\\\"\\\"\\\"\\\"\\n        # Check if there's enough data from the past data for a momentum strategy\\n        try:\\n            self.start_date_idx = np.argwhere(\\n                self.only_month_end_df.index\\n                == datetime.strptime(self.start_date, self.date_format)\\n            ).flatten()[0]\\n\\n            self.end_date_idx = np.argwhere(\\n                self.only_month_end_df.index\\n                == datetime.strptime(self.end_date, self.date_format)\\n            ).flatten()[0]\\n        except:\\n            raise ValueError(f\\\"{start_date} not available in asset history data.\\\")\\n        assert self.start_date_idx - 1 >= self.momentum_signal_period, print(\\n            f\\\"There is not enough data (Number of months < {self.momentum_signal_period}) before {self.start_date} to generate a momentum signal.\\\"\\n        )\\n\\n    def execute_backtest(\\n        self,\\n        start_date: str = \\\"2001-12-31\\\",\\n        end_date: str = \\\"2020-08-31\\\",\\n        date_format: str = \\\"%Y-%m-%d\\\",\\n    ):\\n        \\\"\\\"\\\"Contains the actual trading logic \\\"\\\"\\\"\\n        self.start_date = start_date\\n        self.end_date = end_date\\n        self.date_format = date_format\\n        self.only_month_end_df = self.filter_month_end(df=self.target_asset_history)\\n        self.check_data()\\n        self.past_12_month_returns = self.only_month_end_df.pct_change(\\n            self.momentum_signal_period\\n        ).iloc[\\n            self.start_date_idx : self.end_date_idx + 1\\n        ]  # Cumulative returns from past 12 months\\n        self.monthly_long_position_returns = (\\n            self.only_month_end_df.iloc[self.start_date_idx - 1 :]\\n            .pct_change(1)\\n            .iloc[1 : self.end_date_idx + 1]\\n        )  # End of Month Monthly Long position returns\\n        self.monthly_returns = (\\n            self.past_12_month_returns.apply(np.sign).shift(1).iloc[1:]\\n            * self.monthly_long_position_returns.iloc[1:]\\n        )\\n        self._results()\\n\\n    def _results(self):\\n        \\\"\\\"\\\"\\\"\\\"\\\"\\n        self.annual_results = self.monthly_returns.groupby(\\n            [self.monthly_returns.index.year]\\n        ).agg([self.calendar_year_return])\\n        self.asset_class_annualized_returns = self.annual_results.apply(\\n            self.annualized_return, axis=0\\n        ).droplevel(-1)\\n        self.asset_class_annualized_returns.name = \\\"Annualized Returns\\\"\\n        self.asset_class_annualized_volatility = (\\n            self.monthly_returns.groupby([self.monthly_returns.index.year])\\n            .agg([self.annualized_volatility])\\n            .apply(self.annualized_volatility, axis=0)\\n        ).droplevel(-1)\\n        self.asset_class_annualized_volatility.name = \\\"Annualized Volatility\\\"\\n        self.asset_class_sharpe_ratio = self.annual_results.apply(\\n            self.sharpe_ratio, axis=0\\n        ).droplevel(-1)\\n        self.asset_class_sharpe_ratio.name = \\\"Sharpe Ratio (assuming $R_f$ = 0)\\\"\\n        display(self.annual_results)\\n        display(\\n            pd.concat(\\n                [\\n                    self.asset_class_annualized_returns,\\n                    self.asset_class_annualized_volatility,\\n                    self.asset_class_sharpe_ratio,\\n                ],\\n                axis=1,\\n            ).transpose()\\n        )\";\n",
       "                var nbb_formatted_code = \"class TimeSeriesMomentumStrategy(TradingStrategy):\\n    \\\"\\\"\\\"Longs (Shorts) 100% for 1 month if asset's own past 12 month returns > 0 (< 0) \\n    Assumes that all transactions happen on the end of month\\n    \\\"\\\"\\\"\\n\\n    def __init__(self, momentum_signal_period: int = 12, **kwargs):\\n        super().__init__(name=\\\"Time Series Momentum Strategy\\\", **kwargs)\\n        self.momentum_signal_period = momentum_signal_period\\n        self.target_asset_history = self.asset_class_history[\\n            self.asset_class_history.columns[\\n                self.asset_class_history.columns.get_level_values(\\\"asset_class\\\")\\n                == asset_class\\n            ]\\n        ]\\n\\n    def filter_month_end(self, df):\\n        \\\"\\\"\\\"Filter out only data of end of month\\\"\\\"\\\"\\n        dates = df.index\\n        only_month_end_df = df[pd.Series(dates, index=dates).dt.is_month_end]\\n        return only_month_end_df\\n\\n    def check_data(self):\\n        \\\"\\\"\\\"\\\"\\\"\\\"\\n        # Check if there's enough data from the past data for a momentum strategy\\n        try:\\n            self.start_date_idx = np.argwhere(\\n                self.only_month_end_df.index\\n                == datetime.strptime(self.start_date, self.date_format)\\n            ).flatten()[0]\\n\\n            self.end_date_idx = np.argwhere(\\n                self.only_month_end_df.index\\n                == datetime.strptime(self.end_date, self.date_format)\\n            ).flatten()[0]\\n        except:\\n            raise ValueError(f\\\"{start_date} not available in asset history data.\\\")\\n        assert self.start_date_idx - 1 >= self.momentum_signal_period, print(\\n            f\\\"There is not enough data (Number of months < {self.momentum_signal_period}) before {self.start_date} to generate a momentum signal.\\\"\\n        )\\n\\n    def execute_backtest(\\n        self,\\n        start_date: str = \\\"2001-12-31\\\",\\n        end_date: str = \\\"2020-08-31\\\",\\n        date_format: str = \\\"%Y-%m-%d\\\",\\n    ):\\n        \\\"\\\"\\\"Contains the actual trading logic \\\"\\\"\\\"\\n        self.start_date = start_date\\n        self.end_date = end_date\\n        self.date_format = date_format\\n        self.only_month_end_df = self.filter_month_end(df=self.target_asset_history)\\n        self.check_data()\\n        self.past_12_month_returns = self.only_month_end_df.pct_change(\\n            self.momentum_signal_period\\n        ).iloc[\\n            self.start_date_idx : self.end_date_idx + 1\\n        ]  # Cumulative returns from past 12 months\\n        self.monthly_long_position_returns = (\\n            self.only_month_end_df.iloc[self.start_date_idx - 1 :]\\n            .pct_change(1)\\n            .iloc[1 : self.end_date_idx + 1]\\n        )  # End of Month Monthly Long position returns\\n        self.monthly_returns = (\\n            self.past_12_month_returns.apply(np.sign).shift(1).iloc[1:]\\n            * self.monthly_long_position_returns.iloc[1:]\\n        )\\n        self._results()\\n\\n    def _results(self):\\n        \\\"\\\"\\\"\\\"\\\"\\\"\\n        self.annual_results = self.monthly_returns.groupby(\\n            [self.monthly_returns.index.year]\\n        ).agg([self.calendar_year_return])\\n        self.asset_class_annualized_returns = self.annual_results.apply(\\n            self.annualized_return, axis=0\\n        ).droplevel(-1)\\n        self.asset_class_annualized_returns.name = \\\"Annualized Returns\\\"\\n        self.asset_class_annualized_volatility = (\\n            self.monthly_returns.groupby([self.monthly_returns.index.year])\\n            .agg([self.annualized_volatility])\\n            .apply(self.annualized_volatility, axis=0)\\n        ).droplevel(-1)\\n        self.asset_class_annualized_volatility.name = \\\"Annualized Volatility\\\"\\n        self.asset_class_sharpe_ratio = self.annual_results.apply(\\n            self.sharpe_ratio, axis=0\\n        ).droplevel(-1)\\n        self.asset_class_sharpe_ratio.name = \\\"Sharpe Ratio (assuming $R_f$ = 0)\\\"\\n        display(self.annual_results)\\n        display(\\n            pd.concat(\\n                [\\n                    self.asset_class_annualized_returns,\\n                    self.asset_class_annualized_volatility,\\n                    self.asset_class_sharpe_ratio,\\n                ],\\n                axis=1,\\n            ).transpose()\\n        )\";\n",
       "                var nbb_cells = Jupyter.notebook.get_cells();\n",
       "                for (var i = 0; i < nbb_cells.length; ++i) {\n",
       "                    if (nbb_cells[i].input_prompt_number == nbb_cell_id) {\n",
       "                        if (nbb_cells[i].get_text() == nbb_unformatted_code) {\n",
       "                             nbb_cells[i].set_text(nbb_formatted_code);\n",
       "                        }\n",
       "                        break;\n",
       "                    }\n",
       "                }\n",
       "            }, 500);\n",
       "            "
      ],
      "text/plain": [
       "<IPython.core.display.Javascript object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "time: 58.3 ms\n"
     ]
    }
   ],
   "source": [
    "class TimeSeriesMomentumStrategy(TradingStrategy):\n",
    "    \"\"\"Longs (Shorts) 100% for 1 month if asset's own past 12 month returns > 0 (< 0) \n",
    "    Assumes that all transactions happen on the end of month\n",
    "    \"\"\"\n",
    "\n",
    "    def __init__(self, momentum_signal_period: int = 12, **kwargs):\n",
    "        super().__init__(name=\"Time Series Momentum Strategy\", **kwargs)\n",
    "        self.momentum_signal_period = momentum_signal_period\n",
    "        self.target_asset_history = self.asset_class_history[\n",
    "            self.asset_class_history.columns[\n",
    "                self.asset_class_history.columns.get_level_values(\"asset_class\")\n",
    "                == asset_class\n",
    "            ]\n",
    "        ]\n",
    "\n",
    "    def filter_month_end(self, df):\n",
    "        \"\"\"Filter out only data of end of month\"\"\"\n",
    "        dates = df.index\n",
    "        only_month_end_df = df[pd.Series(dates, index=dates).dt.is_month_end]\n",
    "        return only_month_end_df\n",
    "\n",
    "    def check_data(self):\n",
    "        \"\"\"\"\"\"\n",
    "        # Check if there's enough data from the past data for a momentum strategy\n",
    "        try:\n",
    "            self.start_date_idx = np.argwhere(\n",
    "                self.only_month_end_df.index\n",
    "                == datetime.strptime(self.start_date, self.date_format)\n",
    "            ).flatten()[0]\n",
    "\n",
    "            self.end_date_idx = np.argwhere(\n",
    "                self.only_month_end_df.index\n",
    "                == datetime.strptime(self.end_date, self.date_format)\n",
    "            ).flatten()[0]\n",
    "        except:\n",
    "            raise ValueError(f\"{start_date} not available in asset history data.\")\n",
    "        assert self.start_date_idx - 1 >= self.momentum_signal_period, print(\n",
    "            f\"There is not enough data (Number of months < {self.momentum_signal_period}) before {self.start_date} to generate a momentum signal.\"\n",
    "        )\n",
    "\n",
    "    def execute_backtest(\n",
    "        self,\n",
    "        start_date: str = \"2001-12-31\",\n",
    "        end_date: str = \"2020-08-31\",\n",
    "        date_format: str = \"%Y-%m-%d\",\n",
    "    ):\n",
    "        \"\"\"Contains the actual trading logic \"\"\"\n",
    "        self.start_date = start_date\n",
    "        self.end_date = end_date\n",
    "        self.date_format = date_format\n",
    "        self.only_month_end_df = self.filter_month_end(df=self.target_asset_history)\n",
    "        self.check_data()\n",
    "        self.past_12_month_returns = self.only_month_end_df.pct_change(\n",
    "            self.momentum_signal_period\n",
    "        ).iloc[\n",
    "            self.start_date_idx : self.end_date_idx + 1\n",
    "        ]  # Cumulative returns from past 12 months\n",
    "        self.monthly_long_position_returns = (\n",
    "            self.only_month_end_df.iloc[self.start_date_idx - 1 :]\n",
    "            .pct_change(1)\n",
    "            .iloc[1 : self.end_date_idx + 1]\n",
    "        )  # End of Month Monthly Long position returns\n",
    "        self.monthly_returns = (\n",
    "            self.past_12_month_returns.apply(np.sign).shift(1).iloc[1:]\n",
    "            * self.monthly_long_position_returns.iloc[1:]\n",
    "        )\n",
    "        self._results()\n",
    "\n",
    "    def _results(self):\n",
    "        \"\"\"\"\"\"\n",
    "        self.annual_results = self.monthly_returns.groupby(\n",
    "            [self.monthly_returns.index.year]\n",
    "        ).agg([self.calendar_year_return])\n",
    "        self.asset_class_annualized_returns = self.annual_results.apply(\n",
    "            self.annualized_return, axis=0\n",
    "        ).droplevel(-1)\n",
    "        self.asset_class_annualized_returns.name = \"Annualized Returns\"\n",
    "        self.asset_class_annualized_volatility = (\n",
    "            self.monthly_returns.groupby([self.monthly_returns.index.year])\n",
    "            .agg([self.annualized_volatility])\n",
    "            .apply(self.annualized_volatility, axis=0)\n",
    "        ).droplevel(-1)\n",
    "        self.asset_class_annualized_volatility.name = \"Annualized Volatility\"\n",
    "        self.asset_class_sharpe_ratio = self.annual_results.apply(\n",
    "            self.sharpe_ratio, axis=0\n",
    "        ).droplevel(-1)\n",
    "        self.asset_class_sharpe_ratio.name = \"Sharpe Ratio (assuming $R_f$ = 0)\"\n",
    "        display(self.annual_results)\n",
    "        display(\n",
    "            pd.concat(\n",
    "                [\n",
    "                    self.asset_class_annualized_returns,\n",
    "                    self.asset_class_annualized_volatility,\n",
    "                    self.asset_class_sharpe_ratio,\n",
    "                ],\n",
    "                axis=1,\n",
    "            ).transpose()\n",
    "        )"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {
    "scrolled": false
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "====================================================================================================\n"
     ]
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr th {\n",
       "        text-align: left;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr:last-of-type th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th colspan=\"15\" halign=\"left\">Equity Idx</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>S&amp;P 500 - SPX</th>\n",
       "      <th>Russell 2K - RTY</th>\n",
       "      <th>NASDAQ 100 - NDX</th>\n",
       "      <th>Hang Seng -  HSI</th>\n",
       "      <th>Shanghai SE - SHCOMP</th>\n",
       "      <th>KOSPI - KOSPI</th>\n",
       "      <th>ASX 200 - AS51</th>\n",
       "      <th>NIKKEI - NKY</th>\n",
       "      <th>FTSE 100 - UKX</th>\n",
       "      <th>DAX - DAX</th>\n",
       "      <th>FTSE MIB - FTSEMIB</th>\n",
       "      <th>CAC  - CAC</th>\n",
       "      <th>MSCI India - M1IN</th>\n",
       "      <th>MSCI Russia - M1RU</th>\n",
       "      <th>MSCI Brazil - M1BR</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Date</th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>2002</th>\n",
       "      <td>0.088434</td>\n",
       "      <td>0.279880</td>\n",
       "      <td>0.387414</td>\n",
       "      <td>0.142797</td>\n",
       "      <td>0.135255</td>\n",
       "      <td>-0.071996</td>\n",
       "      <td>0.051451</td>\n",
       "      <td>0.169137</td>\n",
       "      <td>0.232031</td>\n",
       "      <td>0.535081</td>\n",
       "      <td>0.264682</td>\n",
       "      <td>0.353556</td>\n",
       "      <td>0.222429</td>\n",
       "      <td>0.154326</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2003</th>\n",
       "      <td>0.019208</td>\n",
       "      <td>-0.086041</td>\n",
       "      <td>-0.014318</td>\n",
       "      <td>0.113109</td>\n",
       "      <td>-0.199595</td>\n",
       "      <td>-0.015477</td>\n",
       "      <td>0.009032</td>\n",
       "      <td>-0.166724</td>\n",
       "      <td>-0.008626</td>\n",
       "      <td>0.003490</td>\n",
       "      <td>0.020381</td>\n",
       "      <td>-0.058143</td>\n",
       "      <td>0.919416</td>\n",
       "      <td>0.140371</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2004</th>\n",
       "      <td>0.108757</td>\n",
       "      <td>0.182753</td>\n",
       "      <td>0.106847</td>\n",
       "      <td>0.171412</td>\n",
       "      <td>-0.074865</td>\n",
       "      <td>0.137561</td>\n",
       "      <td>0.295137</td>\n",
       "      <td>0.086359</td>\n",
       "      <td>0.112444</td>\n",
       "      <td>0.073369</td>\n",
       "      <td>0.188540</td>\n",
       "      <td>0.101350</td>\n",
       "      <td>0.191084</td>\n",
       "      <td>0.054620</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2005</th>\n",
       "      <td>0.049069</td>\n",
       "      <td>0.045117</td>\n",
       "      <td>-0.123204</td>\n",
       "      <td>0.084047</td>\n",
       "      <td>0.018237</td>\n",
       "      <td>0.567997</td>\n",
       "      <td>0.240955</td>\n",
       "      <td>0.278645</td>\n",
       "      <td>0.208053</td>\n",
       "      <td>0.270714</td>\n",
       "      <td>0.201198</td>\n",
       "      <td>0.269533</td>\n",
       "      <td>0.375694</td>\n",
       "      <td>0.700018</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2006</th>\n",
       "      <td>0.157767</td>\n",
       "      <td>0.183298</td>\n",
       "      <td>-0.024914</td>\n",
       "      <td>0.389660</td>\n",
       "      <td>0.988308</td>\n",
       "      <td>0.060301</td>\n",
       "      <td>0.254948</td>\n",
       "      <td>0.080900</td>\n",
       "      <td>0.144439</td>\n",
       "      <td>0.219786</td>\n",
       "      <td>0.209513</td>\n",
       "      <td>0.209697</td>\n",
       "      <td>0.510044</td>\n",
       "      <td>0.555970</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2007</th>\n",
       "      <td>0.055713</td>\n",
       "      <td>-0.014266</td>\n",
       "      <td>0.191648</td>\n",
       "      <td>0.433789</td>\n",
       "      <td>0.979663</td>\n",
       "      <td>0.237113</td>\n",
       "      <td>0.174193</td>\n",
       "      <td>-0.058043</td>\n",
       "      <td>0.073884</td>\n",
       "      <td>0.222892</td>\n",
       "      <td>-0.031965</td>\n",
       "      <td>0.041919</td>\n",
       "      <td>0.731106</td>\n",
       "      <td>0.245037</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2008</th>\n",
       "      <td>0.330330</td>\n",
       "      <td>0.385477</td>\n",
       "      <td>0.202456</td>\n",
       "      <td>0.055844</td>\n",
       "      <td>0.227160</td>\n",
       "      <td>0.213187</td>\n",
       "      <td>0.136177</td>\n",
       "      <td>0.516562</td>\n",
       "      <td>0.108632</td>\n",
       "      <td>0.048457</td>\n",
       "      <td>0.730014</td>\n",
       "      <td>0.198327</td>\n",
       "      <td>-0.159019</td>\n",
       "      <td>0.501275</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2009</th>\n",
       "      <td>-0.119551</td>\n",
       "      <td>-0.097311</td>\n",
       "      <td>-0.273061</td>\n",
       "      <td>-0.360089</td>\n",
       "      <td>-0.359757</td>\n",
       "      <td>-0.290330</td>\n",
       "      <td>-0.251316</td>\n",
       "      <td>-0.120017</td>\n",
       "      <td>-0.145776</td>\n",
       "      <td>-0.091792</td>\n",
       "      <td>-0.184650</td>\n",
       "      <td>-0.107100</td>\n",
       "      <td>-0.545699</td>\n",
       "      <td>-0.578876</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2010</th>\n",
       "      <td>0.150587</td>\n",
       "      <td>0.268146</td>\n",
       "      <td>0.200588</td>\n",
       "      <td>0.085812</td>\n",
       "      <td>-0.441488</td>\n",
       "      <td>0.235763</td>\n",
       "      <td>0.029409</td>\n",
       "      <td>-0.148346</td>\n",
       "      <td>0.126689</td>\n",
       "      <td>0.160599</td>\n",
       "      <td>-0.327295</td>\n",
       "      <td>-0.115950</td>\n",
       "      <td>0.209516</td>\n",
       "      <td>0.190655</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2011</th>\n",
       "      <td>0.021055</td>\n",
       "      <td>-0.293698</td>\n",
       "      <td>0.036123</td>\n",
       "      <td>-0.271779</td>\n",
       "      <td>-0.011050</td>\n",
       "      <td>-0.228118</td>\n",
       "      <td>-0.140106</td>\n",
       "      <td>-0.104074</td>\n",
       "      <td>-0.169795</td>\n",
       "      <td>-0.193300</td>\n",
       "      <td>-0.152196</td>\n",
       "      <td>-0.111474</td>\n",
       "      <td>-0.061607</td>\n",
       "      <td>-0.317564</td>\n",
       "      <td>-0.534078</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2012</th>\n",
       "      <td>0.068127</td>\n",
       "      <td>0.051368</td>\n",
       "      <td>0.183478</td>\n",
       "      <td>-0.103584</td>\n",
       "      <td>-0.094842</td>\n",
       "      <td>-0.012624</td>\n",
       "      <td>0.026742</td>\n",
       "      <td>-0.029484</td>\n",
       "      <td>0.066475</td>\n",
       "      <td>-0.097605</td>\n",
       "      <td>0.004187</td>\n",
       "      <td>-0.053662</td>\n",
       "      <td>-0.332252</td>\n",
       "      <td>-0.249583</td>\n",
       "      <td>-0.079200</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2013</th>\n",
       "      <td>0.323742</td>\n",
       "      <td>0.388219</td>\n",
       "      <td>0.369096</td>\n",
       "      <td>0.065616</td>\n",
       "      <td>0.069251</td>\n",
       "      <td>0.019623</td>\n",
       "      <td>0.219773</td>\n",
       "      <td>0.593699</td>\n",
       "      <td>0.186501</td>\n",
       "      <td>0.254817</td>\n",
       "      <td>-0.006969</td>\n",
       "      <td>0.222258</td>\n",
       "      <td>-0.396076</td>\n",
       "      <td>-0.092174</td>\n",
       "      <td>-0.058150</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2014</th>\n",
       "      <td>0.136783</td>\n",
       "      <td>0.048962</td>\n",
       "      <td>0.193963</td>\n",
       "      <td>-0.021945</td>\n",
       "      <td>0.624548</td>\n",
       "      <td>-0.016952</td>\n",
       "      <td>0.071487</td>\n",
       "      <td>0.089566</td>\n",
       "      <td>0.007179</td>\n",
       "      <td>0.026527</td>\n",
       "      <td>0.029781</td>\n",
       "      <td>0.025418</td>\n",
       "      <td>0.048017</td>\n",
       "      <td>0.137143</td>\n",
       "      <td>-0.070510</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2015</th>\n",
       "      <td>-0.143971</td>\n",
       "      <td>-0.044132</td>\n",
       "      <td>0.097507</td>\n",
       "      <td>-0.086906</td>\n",
       "      <td>0.111542</td>\n",
       "      <td>0.002824</td>\n",
       "      <td>0.100783</td>\n",
       "      <td>0.109935</td>\n",
       "      <td>-0.024950</td>\n",
       "      <td>0.095605</td>\n",
       "      <td>0.157532</td>\n",
       "      <td>0.119448</td>\n",
       "      <td>-0.055465</td>\n",
       "      <td>-0.141711</td>\n",
       "      <td>0.516565</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2016</th>\n",
       "      <td>-0.020077</td>\n",
       "      <td>-0.020325</td>\n",
       "      <td>-0.143435</td>\n",
       "      <td>-0.130668</td>\n",
       "      <td>-0.303595</td>\n",
       "      <td>-0.057955</td>\n",
       "      <td>-0.028356</td>\n",
       "      <td>-0.324395</td>\n",
       "      <td>0.043638</td>\n",
       "      <td>-0.225592</td>\n",
       "      <td>-0.217988</td>\n",
       "      <td>-0.150836</td>\n",
       "      <td>-0.182770</td>\n",
       "      <td>0.186032</td>\n",
       "      <td>-0.466803</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2017</th>\n",
       "      <td>0.218228</td>\n",
       "      <td>0.146277</td>\n",
       "      <td>0.329921</td>\n",
       "      <td>0.412730</td>\n",
       "      <td>0.049283</td>\n",
       "      <td>0.239290</td>\n",
       "      <td>0.134067</td>\n",
       "      <td>0.213360</td>\n",
       "      <td>0.119541</td>\n",
       "      <td>0.125126</td>\n",
       "      <td>0.244154</td>\n",
       "      <td>0.125445</td>\n",
       "      <td>0.271831</td>\n",
       "      <td>0.052043</td>\n",
       "      <td>0.241117</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2018</th>\n",
       "      <td>-0.043918</td>\n",
       "      <td>-0.110274</td>\n",
       "      <td>0.000368</td>\n",
       "      <td>-0.170337</td>\n",
       "      <td>-0.044080</td>\n",
       "      <td>0.060109</td>\n",
       "      <td>-0.014635</td>\n",
       "      <td>-0.103034</td>\n",
       "      <td>0.009453</td>\n",
       "      <td>-0.031980</td>\n",
       "      <td>0.036292</td>\n",
       "      <td>0.054855</td>\n",
       "      <td>-0.245235</td>\n",
       "      <td>-0.003945</td>\n",
       "      <td>-0.524253</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2019</th>\n",
       "      <td>0.050027</td>\n",
       "      <td>-0.222140</td>\n",
       "      <td>0.394574</td>\n",
       "      <td>-0.177570</td>\n",
       "      <td>-0.254777</td>\n",
       "      <td>0.020049</td>\n",
       "      <td>0.156804</td>\n",
       "      <td>-0.174382</td>\n",
       "      <td>-0.038051</td>\n",
       "      <td>-0.139648</td>\n",
       "      <td>-0.089326</td>\n",
       "      <td>-0.078987</td>\n",
       "      <td>-0.074335</td>\n",
       "      <td>0.128606</td>\n",
       "      <td>-0.065489</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2020</th>\n",
       "      <td>-0.219109</td>\n",
       "      <td>-0.324053</td>\n",
       "      <td>0.648726</td>\n",
       "      <td>-0.103680</td>\n",
       "      <td>0.079134</td>\n",
       "      <td>0.181846</td>\n",
       "      <td>-0.497120</td>\n",
       "      <td>-0.374425</td>\n",
       "      <td>-0.107319</td>\n",
       "      <td>-0.505415</td>\n",
       "      <td>-0.514804</td>\n",
       "      <td>-0.486342</td>\n",
       "      <td>-0.668994</td>\n",
       "      <td>-0.635793</td>\n",
       "      <td>0.124278</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "               Equity Idx                                            \\\n",
       "            S&P 500 - SPX     Russell 2K - RTY     NASDAQ 100 - NDX   \n",
       "     calendar_year_return calendar_year_return calendar_year_return   \n",
       "Date                                                                  \n",
       "2002             0.088434             0.279880             0.387414   \n",
       "2003             0.019208            -0.086041            -0.014318   \n",
       "2004             0.108757             0.182753             0.106847   \n",
       "2005             0.049069             0.045117            -0.123204   \n",
       "2006             0.157767             0.183298            -0.024914   \n",
       "2007             0.055713            -0.014266             0.191648   \n",
       "2008             0.330330             0.385477             0.202456   \n",
       "2009            -0.119551            -0.097311            -0.273061   \n",
       "2010             0.150587             0.268146             0.200588   \n",
       "2011             0.021055            -0.293698             0.036123   \n",
       "2012             0.068127             0.051368             0.183478   \n",
       "2013             0.323742             0.388219             0.369096   \n",
       "2014             0.136783             0.048962             0.193963   \n",
       "2015            -0.143971            -0.044132             0.097507   \n",
       "2016            -0.020077            -0.020325            -0.143435   \n",
       "2017             0.218228             0.146277             0.329921   \n",
       "2018            -0.043918            -0.110274             0.000368   \n",
       "2019             0.050027            -0.222140             0.394574   \n",
       "2020            -0.219109            -0.324053             0.648726   \n",
       "\n",
       "                                                                     \\\n",
       "        Hang Seng -  HSI  Shanghai SE - SHCOMP        KOSPI - KOSPI   \n",
       "     calendar_year_return calendar_year_return calendar_year_return   \n",
       "Date                                                                  \n",
       "2002             0.142797             0.135255            -0.071996   \n",
       "2003             0.113109            -0.199595            -0.015477   \n",
       "2004             0.171412            -0.074865             0.137561   \n",
       "2005             0.084047             0.018237             0.567997   \n",
       "2006             0.389660             0.988308             0.060301   \n",
       "2007             0.433789             0.979663             0.237113   \n",
       "2008             0.055844             0.227160             0.213187   \n",
       "2009            -0.360089            -0.359757            -0.290330   \n",
       "2010             0.085812            -0.441488             0.235763   \n",
       "2011            -0.271779            -0.011050            -0.228118   \n",
       "2012            -0.103584            -0.094842            -0.012624   \n",
       "2013             0.065616             0.069251             0.019623   \n",
       "2014            -0.021945             0.624548            -0.016952   \n",
       "2015            -0.086906             0.111542             0.002824   \n",
       "2016            -0.130668            -0.303595            -0.057955   \n",
       "2017             0.412730             0.049283             0.239290   \n",
       "2018            -0.170337            -0.044080             0.060109   \n",
       "2019            -0.177570            -0.254777             0.020049   \n",
       "2020            -0.103680             0.079134             0.181846   \n",
       "\n",
       "                                                                     \\\n",
       "           ASX 200 - AS51         NIKKEI - NKY       FTSE 100 - UKX   \n",
       "     calendar_year_return calendar_year_return calendar_year_return   \n",
       "Date                                                                  \n",
       "2002             0.051451             0.169137             0.232031   \n",
       "2003             0.009032            -0.166724            -0.008626   \n",
       "2004             0.295137             0.086359             0.112444   \n",
       "2005             0.240955             0.278645             0.208053   \n",
       "2006             0.254948             0.080900             0.144439   \n",
       "2007             0.174193            -0.058043             0.073884   \n",
       "2008             0.136177             0.516562             0.108632   \n",
       "2009            -0.251316            -0.120017            -0.145776   \n",
       "2010             0.029409            -0.148346             0.126689   \n",
       "2011            -0.140106            -0.104074            -0.169795   \n",
       "2012             0.026742            -0.029484             0.066475   \n",
       "2013             0.219773             0.593699             0.186501   \n",
       "2014             0.071487             0.089566             0.007179   \n",
       "2015             0.100783             0.109935            -0.024950   \n",
       "2016            -0.028356            -0.324395             0.043638   \n",
       "2017             0.134067             0.213360             0.119541   \n",
       "2018            -0.014635            -0.103034             0.009453   \n",
       "2019             0.156804            -0.174382            -0.038051   \n",
       "2020            -0.497120            -0.374425            -0.107319   \n",
       "\n",
       "                                                                     \\\n",
       "                DAX - DAX   FTSE MIB - FTSEMIB           CAC  - CAC   \n",
       "     calendar_year_return calendar_year_return calendar_year_return   \n",
       "Date                                                                  \n",
       "2002             0.535081             0.264682             0.353556   \n",
       "2003             0.003490             0.020381            -0.058143   \n",
       "2004             0.073369             0.188540             0.101350   \n",
       "2005             0.270714             0.201198             0.269533   \n",
       "2006             0.219786             0.209513             0.209697   \n",
       "2007             0.222892            -0.031965             0.041919   \n",
       "2008             0.048457             0.730014             0.198327   \n",
       "2009            -0.091792            -0.184650            -0.107100   \n",
       "2010             0.160599            -0.327295            -0.115950   \n",
       "2011            -0.193300            -0.152196            -0.111474   \n",
       "2012            -0.097605             0.004187            -0.053662   \n",
       "2013             0.254817            -0.006969             0.222258   \n",
       "2014             0.026527             0.029781             0.025418   \n",
       "2015             0.095605             0.157532             0.119448   \n",
       "2016            -0.225592            -0.217988            -0.150836   \n",
       "2017             0.125126             0.244154             0.125445   \n",
       "2018            -0.031980             0.036292             0.054855   \n",
       "2019            -0.139648            -0.089326            -0.078987   \n",
       "2020            -0.505415            -0.514804            -0.486342   \n",
       "\n",
       "                                                                     \n",
       "        MSCI India - M1IN   MSCI Russia - M1RU   MSCI Brazil - M1BR  \n",
       "     calendar_year_return calendar_year_return calendar_year_return  \n",
       "Date                                                                 \n",
       "2002             0.222429             0.154326                  NaN  \n",
       "2003             0.919416             0.140371                  NaN  \n",
       "2004             0.191084             0.054620                  NaN  \n",
       "2005             0.375694             0.700018                  NaN  \n",
       "2006             0.510044             0.555970                  NaN  \n",
       "2007             0.731106             0.245037                  NaN  \n",
       "2008            -0.159019             0.501275                  NaN  \n",
       "2009            -0.545699            -0.578876                  NaN  \n",
       "2010             0.209516             0.190655                  NaN  \n",
       "2011            -0.061607            -0.317564            -0.534078  \n",
       "2012            -0.332252            -0.249583            -0.079200  \n",
       "2013            -0.396076            -0.092174            -0.058150  \n",
       "2014             0.048017             0.137143            -0.070510  \n",
       "2015            -0.055465            -0.141711             0.516565  \n",
       "2016            -0.182770             0.186032            -0.466803  \n",
       "2017             0.271831             0.052043             0.241117  \n",
       "2018            -0.245235            -0.003945            -0.524253  \n",
       "2019            -0.074335             0.128606            -0.065489  \n",
       "2020            -0.668994            -0.635793             0.124278  "
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr th {\n",
       "        text-align: left;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th colspan=\"15\" halign=\"left\">Equity Idx</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>S&amp;P 500 - SPX</th>\n",
       "      <th>Russell 2K - RTY</th>\n",
       "      <th>NASDAQ 100 - NDX</th>\n",
       "      <th>Hang Seng -  HSI</th>\n",
       "      <th>Shanghai SE - SHCOMP</th>\n",
       "      <th>KOSPI - KOSPI</th>\n",
       "      <th>ASX 200 - AS51</th>\n",
       "      <th>NIKKEI - NKY</th>\n",
       "      <th>FTSE 100 - UKX</th>\n",
       "      <th>DAX - DAX</th>\n",
       "      <th>FTSE MIB - FTSEMIB</th>\n",
       "      <th>CAC  - CAC</th>\n",
       "      <th>MSCI India - M1IN</th>\n",
       "      <th>MSCI Russia - M1RU</th>\n",
       "      <th>MSCI Brazil - M1BR</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>Annualized Returns</th>\n",
       "      <td>0.055576</td>\n",
       "      <td>0.019841</td>\n",
       "      <td>0.124717</td>\n",
       "      <td>0.005114</td>\n",
       "      <td>0.019207</td>\n",
       "      <td>0.051567</td>\n",
       "      <td>0.030362</td>\n",
       "      <td>-0.000177</td>\n",
       "      <td>0.043581</td>\n",
       "      <td>0.012956</td>\n",
       "      <td>-0.004439</td>\n",
       "      <td>0.009452</td>\n",
       "      <td>-0.044851</td>\n",
       "      <td>-0.011820</td>\n",
       "      <td>-0.153728</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Annualized Volatility</th>\n",
       "      <td>0.250217</td>\n",
       "      <td>0.258396</td>\n",
       "      <td>0.299556</td>\n",
       "      <td>0.279455</td>\n",
       "      <td>0.396346</td>\n",
       "      <td>0.309805</td>\n",
       "      <td>0.180394</td>\n",
       "      <td>0.215803</td>\n",
       "      <td>0.201919</td>\n",
       "      <td>0.342666</td>\n",
       "      <td>0.298922</td>\n",
       "      <td>0.263166</td>\n",
       "      <td>0.399527</td>\n",
       "      <td>0.422950</td>\n",
       "      <td>0.262285</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Sharpe Ratio (assuming $R_f$ = 0)</th>\n",
       "      <td>0.091799</td>\n",
       "      <td>0.022376</td>\n",
       "      <td>0.132030</td>\n",
       "      <td>0.005419</td>\n",
       "      <td>0.011392</td>\n",
       "      <td>0.063658</td>\n",
       "      <td>0.037631</td>\n",
       "      <td>-0.000164</td>\n",
       "      <td>0.089658</td>\n",
       "      <td>0.013405</td>\n",
       "      <td>-0.003924</td>\n",
       "      <td>0.011559</td>\n",
       "      <td>-0.025470</td>\n",
       "      <td>-0.008077</td>\n",
       "      <td>-0.149786</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                                     Equity Idx                   \\\n",
       "                                  S&P 500 - SPX Russell 2K - RTY   \n",
       "Annualized Returns                     0.055576         0.019841   \n",
       "Annualized Volatility                  0.250217         0.258396   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)      0.091799         0.022376   \n",
       "\n",
       "                                                                      \\\n",
       "                                  NASDAQ 100 - NDX Hang Seng -  HSI    \n",
       "Annualized Returns                        0.124717          0.005114   \n",
       "Annualized Volatility                     0.299556          0.279455   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)         0.132030          0.005419   \n",
       "\n",
       "                                                                      \\\n",
       "                                  Shanghai SE - SHCOMP KOSPI - KOSPI   \n",
       "Annualized Returns                            0.019207      0.051567   \n",
       "Annualized Volatility                         0.396346      0.309805   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)             0.011392      0.063658   \n",
       "\n",
       "                                                                              \\\n",
       "                                  ASX 200 - AS51 NIKKEI - NKY FTSE 100 - UKX   \n",
       "Annualized Returns                      0.030362    -0.000177       0.043581   \n",
       "Annualized Volatility                   0.180394     0.215803       0.201919   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)       0.037631    -0.000164       0.089658   \n",
       "\n",
       "                                                                           \\\n",
       "                                  DAX - DAX FTSE MIB - FTSEMIB CAC  - CAC   \n",
       "Annualized Returns                 0.012956          -0.004439   0.009452   \n",
       "Annualized Volatility              0.342666           0.298922   0.263166   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)  0.013405          -0.003924   0.011559   \n",
       "\n",
       "                                                                        \\\n",
       "                                  MSCI India - M1IN MSCI Russia - M1RU   \n",
       "Annualized Returns                        -0.044851          -0.011820   \n",
       "Annualized Volatility                      0.399527           0.422950   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)         -0.025470          -0.008077   \n",
       "\n",
       "                                                      \n",
       "                                  MSCI Brazil - M1BR  \n",
       "Annualized Returns                         -0.153728  \n",
       "Annualized Volatility                       0.262285  \n",
       "Sharpe Ratio (assuming $R_f$ = 0)          -0.149786  "
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "====================================================================================================\n"
     ]
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr th {\n",
       "        text-align: left;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr:last-of-type th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th colspan=\"5\" halign=\"left\">Credit</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>US IG - LUACTRUU</th>\n",
       "      <th>US HY - H0A0</th>\n",
       "      <th>EU IG - SPEZICUT</th>\n",
       "      <th>EU HY - HE00</th>\n",
       "      <th>EM Corp - EMCB</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Date</th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>2002</th>\n",
       "      <td>0.101171</td>\n",
       "      <td>-0.114729</td>\n",
       "      <td>NaN</td>\n",
       "      <td>0.033558</td>\n",
       "      <td>0.107632</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2003</th>\n",
       "      <td>0.082424</td>\n",
       "      <td>0.206790</td>\n",
       "      <td>NaN</td>\n",
       "      <td>0.130784</td>\n",
       "      <td>0.171954</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2004</th>\n",
       "      <td>0.044620</td>\n",
       "      <td>0.108698</td>\n",
       "      <td>NaN</td>\n",
       "      <td>0.145615</td>\n",
       "      <td>0.100654</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2005</th>\n",
       "      <td>0.016762</td>\n",
       "      <td>0.027381</td>\n",
       "      <td>NaN</td>\n",
       "      <td>0.059634</td>\n",
       "      <td>0.043976</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2006</th>\n",
       "      <td>0.013826</td>\n",
       "      <td>0.117671</td>\n",
       "      <td>NaN</td>\n",
       "      <td>0.111028</td>\n",
       "      <td>0.076199</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2007</th>\n",
       "      <td>0.045608</td>\n",
       "      <td>0.021929</td>\n",
       "      <td>NaN</td>\n",
       "      <td>-0.022205</td>\n",
       "      <td>0.049376</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2008</th>\n",
       "      <td>-0.129893</td>\n",
       "      <td>0.258021</td>\n",
       "      <td>NaN</td>\n",
       "      <td>0.436900</td>\n",
       "      <td>0.013024</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2009</th>\n",
       "      <td>0.064239</td>\n",
       "      <td>-0.188552</td>\n",
       "      <td>NaN</td>\n",
       "      <td>-0.256437</td>\n",
       "      <td>-0.155023</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2010</th>\n",
       "      <td>0.089958</td>\n",
       "      <td>0.151904</td>\n",
       "      <td>NaN</td>\n",
       "      <td>0.142566</td>\n",
       "      <td>0.114395</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2011</th>\n",
       "      <td>0.081460</td>\n",
       "      <td>0.043827</td>\n",
       "      <td>NaN</td>\n",
       "      <td>-0.110336</td>\n",
       "      <td>-0.056776</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2012</th>\n",
       "      <td>0.098174</td>\n",
       "      <td>0.155834</td>\n",
       "      <td>NaN</td>\n",
       "      <td>0.115463</td>\n",
       "      <td>0.157965</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2013</th>\n",
       "      <td>-0.036943</td>\n",
       "      <td>0.074187</td>\n",
       "      <td>NaN</td>\n",
       "      <td>0.100641</td>\n",
       "      <td>-0.005148</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2014</th>\n",
       "      <td>0.036368</td>\n",
       "      <td>0.025033</td>\n",
       "      <td>NaN</td>\n",
       "      <td>0.054770</td>\n",
       "      <td>-0.016850</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2015</th>\n",
       "      <td>-0.021545</td>\n",
       "      <td>0.060394</td>\n",
       "      <td>NaN</td>\n",
       "      <td>0.007552</td>\n",
       "      <td>0.051350</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2016</th>\n",
       "      <td>-0.019228</td>\n",
       "      <td>-0.020808</td>\n",
       "      <td>-0.066869</td>\n",
       "      <td>-0.012205</td>\n",
       "      <td>0.092452</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2017</th>\n",
       "      <td>0.064178</td>\n",
       "      <td>0.074832</td>\n",
       "      <td>0.025810</td>\n",
       "      <td>0.067444</td>\n",
       "      <td>0.081342</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2018</th>\n",
       "      <td>-0.040784</td>\n",
       "      <td>-0.022649</td>\n",
       "      <td>0.000440</td>\n",
       "      <td>0.010756</td>\n",
       "      <td>-0.030833</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2019</th>\n",
       "      <td>0.092737</td>\n",
       "      <td>0.043631</td>\n",
       "      <td>-0.091023</td>\n",
       "      <td>0.025470</td>\n",
       "      <td>0.069415</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2020</th>\n",
       "      <td>0.105935</td>\n",
       "      <td>-0.318564</td>\n",
       "      <td>-0.265735</td>\n",
       "      <td>-0.364253</td>\n",
       "      <td>-0.034163</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                   Credit                                            \\\n",
       "         US IG - LUACTRUU         US HY - H0A0     EU IG - SPEZICUT   \n",
       "     calendar_year_return calendar_year_return calendar_year_return   \n",
       "Date                                                                  \n",
       "2002             0.101171            -0.114729                  NaN   \n",
       "2003             0.082424             0.206790                  NaN   \n",
       "2004             0.044620             0.108698                  NaN   \n",
       "2005             0.016762             0.027381                  NaN   \n",
       "2006             0.013826             0.117671                  NaN   \n",
       "2007             0.045608             0.021929                  NaN   \n",
       "2008            -0.129893             0.258021                  NaN   \n",
       "2009             0.064239            -0.188552                  NaN   \n",
       "2010             0.089958             0.151904                  NaN   \n",
       "2011             0.081460             0.043827                  NaN   \n",
       "2012             0.098174             0.155834                  NaN   \n",
       "2013            -0.036943             0.074187                  NaN   \n",
       "2014             0.036368             0.025033                  NaN   \n",
       "2015            -0.021545             0.060394                  NaN   \n",
       "2016            -0.019228            -0.020808            -0.066869   \n",
       "2017             0.064178             0.074832             0.025810   \n",
       "2018            -0.040784            -0.022649             0.000440   \n",
       "2019             0.092737             0.043631            -0.091023   \n",
       "2020             0.105935            -0.318564            -0.265735   \n",
       "\n",
       "                                                \n",
       "             EU HY - HE00       EM Corp - EMCB  \n",
       "     calendar_year_return calendar_year_return  \n",
       "Date                                            \n",
       "2002             0.033558             0.107632  \n",
       "2003             0.130784             0.171954  \n",
       "2004             0.145615             0.100654  \n",
       "2005             0.059634             0.043976  \n",
       "2006             0.111028             0.076199  \n",
       "2007            -0.022205             0.049376  \n",
       "2008             0.436900             0.013024  \n",
       "2009            -0.256437            -0.155023  \n",
       "2010             0.142566             0.114395  \n",
       "2011            -0.110336            -0.056776  \n",
       "2012             0.115463             0.157965  \n",
       "2013             0.100641            -0.005148  \n",
       "2014             0.054770            -0.016850  \n",
       "2015             0.007552             0.051350  \n",
       "2016            -0.012205             0.092452  \n",
       "2017             0.067444             0.081342  \n",
       "2018             0.010756            -0.030833  \n",
       "2019             0.025470             0.069415  \n",
       "2020            -0.364253            -0.034163  "
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr th {\n",
       "        text-align: left;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th colspan=\"5\" halign=\"left\">Credit</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>US IG - LUACTRUU</th>\n",
       "      <th>US HY - H0A0</th>\n",
       "      <th>EU IG - SPEZICUT</th>\n",
       "      <th>EU HY - HE00</th>\n",
       "      <th>EM Corp - EMCB</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>Annualized Returns</th>\n",
       "      <td>0.034379</td>\n",
       "      <td>0.027816</td>\n",
       "      <td>-0.085631</td>\n",
       "      <td>0.022193</td>\n",
       "      <td>0.040705</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Annualized Volatility</th>\n",
       "      <td>0.110831</td>\n",
       "      <td>0.211764</td>\n",
       "      <td>0.021668</td>\n",
       "      <td>0.250086</td>\n",
       "      <td>0.209394</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Sharpe Ratio (assuming $R_f$ = 0)</th>\n",
       "      <td>0.128956</td>\n",
       "      <td>0.048584</td>\n",
       "      <td>-0.373882</td>\n",
       "      <td>0.031851</td>\n",
       "      <td>0.119703</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                                            Credit               \\\n",
       "                                  US IG - LUACTRUU US HY - H0A0   \n",
       "Annualized Returns                        0.034379     0.027816   \n",
       "Annualized Volatility                     0.110831     0.211764   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)         0.128956     0.048584   \n",
       "\n",
       "                                                                                \n",
       "                                  EU IG - SPEZICUT EU HY - HE00 EM Corp - EMCB  \n",
       "Annualized Returns                       -0.085631     0.022193       0.040705  \n",
       "Annualized Volatility                     0.021668     0.250086       0.209394  \n",
       "Sharpe Ratio (assuming $R_f$ = 0)        -0.373882     0.031851       0.119703  "
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "====================================================================================================\n"
     ]
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr th {\n",
       "        text-align: left;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr:last-of-type th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th colspan=\"5\" halign=\"left\">Govt Bond</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>US Treasury - LUATTRUU</th>\n",
       "      <th>UK Gilt - JFBG3GUS</th>\n",
       "      <th>German Bund - CBKIG0FT</th>\n",
       "      <th>Japan JGB - SPJGBTR</th>\n",
       "      <th>EM Govt - GBIEMCOR</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Date</th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>2002</th>\n",
       "      <td>0.117935</td>\n",
       "      <td>0.066458</td>\n",
       "      <td>0.105547</td>\n",
       "      <td>0.051543</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2003</th>\n",
       "      <td>0.022414</td>\n",
       "      <td>0.006415</td>\n",
       "      <td>0.020399</td>\n",
       "      <td>-0.041043</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2004</th>\n",
       "      <td>0.007724</td>\n",
       "      <td>0.003808</td>\n",
       "      <td>0.042187</td>\n",
       "      <td>0.005642</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2005</th>\n",
       "      <td>0.027919</td>\n",
       "      <td>0.028276</td>\n",
       "      <td>0.042376</td>\n",
       "      <td>0.006645</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2006</th>\n",
       "      <td>-0.001478</td>\n",
       "      <td>-0.030992</td>\n",
       "      <td>-0.051699</td>\n",
       "      <td>-0.013907</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2007</th>\n",
       "      <td>0.090124</td>\n",
       "      <td>-0.005503</td>\n",
       "      <td>0.020445</td>\n",
       "      <td>0.040500</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2008</th>\n",
       "      <td>0.137371</td>\n",
       "      <td>0.104842</td>\n",
       "      <td>-0.027833</td>\n",
       "      <td>0.035918</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2009</th>\n",
       "      <td>-0.035674</td>\n",
       "      <td>-0.032949</td>\n",
       "      <td>-0.000288</td>\n",
       "      <td>0.025105</td>\n",
       "      <td>-0.088812</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2010</th>\n",
       "      <td>0.004481</td>\n",
       "      <td>0.047323</td>\n",
       "      <td>0.027167</td>\n",
       "      <td>0.024459</td>\n",
       "      <td>0.154323</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2011</th>\n",
       "      <td>0.098133</td>\n",
       "      <td>0.150250</td>\n",
       "      <td>0.052122</td>\n",
       "      <td>0.027175</td>\n",
       "      <td>-0.125680</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2012</th>\n",
       "      <td>0.019934</td>\n",
       "      <td>0.051032</td>\n",
       "      <td>0.067543</td>\n",
       "      <td>0.018526</td>\n",
       "      <td>-0.179821</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2013</th>\n",
       "      <td>0.007119</td>\n",
       "      <td>0.053977</td>\n",
       "      <td>-0.021530</td>\n",
       "      <td>0.004020</td>\n",
       "      <td>-0.070201</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2014</th>\n",
       "      <td>-0.007403</td>\n",
       "      <td>0.050203</td>\n",
       "      <td>0.077912</td>\n",
       "      <td>0.040626</td>\n",
       "      <td>-0.063224</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2015</th>\n",
       "      <td>0.008425</td>\n",
       "      <td>0.005043</td>\n",
       "      <td>0.020838</td>\n",
       "      <td>0.014520</td>\n",
       "      <td>0.155773</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2016</th>\n",
       "      <td>0.010376</td>\n",
       "      <td>0.038564</td>\n",
       "      <td>0.070891</td>\n",
       "      <td>0.010483</td>\n",
       "      <td>-0.165256</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2017</th>\n",
       "      <td>-0.015598</td>\n",
       "      <td>0.037761</td>\n",
       "      <td>-0.047833</td>\n",
       "      <td>-0.013668</td>\n",
       "      <td>0.146637</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2018</th>\n",
       "      <td>-0.065910</td>\n",
       "      <td>-0.012898</td>\n",
       "      <td>0.004869</td>\n",
       "      <td>0.014964</td>\n",
       "      <td>-0.024602</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2019</th>\n",
       "      <td>0.068576</td>\n",
       "      <td>0.047258</td>\n",
       "      <td>0.048424</td>\n",
       "      <td>-0.003778</td>\n",
       "      <td>-0.063896</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2020</th>\n",
       "      <td>0.134123</td>\n",
       "      <td>0.074121</td>\n",
       "      <td>0.042626</td>\n",
       "      <td>-0.024609</td>\n",
       "      <td>-0.350187</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                  Govt Bond                                              \\\n",
       "     US Treasury - LUATTRUU   UK Gilt - JFBG3GUS German Bund - CBKIG0FT   \n",
       "       calendar_year_return calendar_year_return   calendar_year_return   \n",
       "Date                                                                      \n",
       "2002               0.117935             0.066458               0.105547   \n",
       "2003               0.022414             0.006415               0.020399   \n",
       "2004               0.007724             0.003808               0.042187   \n",
       "2005               0.027919             0.028276               0.042376   \n",
       "2006              -0.001478            -0.030992              -0.051699   \n",
       "2007               0.090124            -0.005503               0.020445   \n",
       "2008               0.137371             0.104842              -0.027833   \n",
       "2009              -0.035674            -0.032949              -0.000288   \n",
       "2010               0.004481             0.047323               0.027167   \n",
       "2011               0.098133             0.150250               0.052122   \n",
       "2012               0.019934             0.051032               0.067543   \n",
       "2013               0.007119             0.053977              -0.021530   \n",
       "2014              -0.007403             0.050203               0.077912   \n",
       "2015               0.008425             0.005043               0.020838   \n",
       "2016               0.010376             0.038564               0.070891   \n",
       "2017              -0.015598             0.037761              -0.047833   \n",
       "2018              -0.065910            -0.012898               0.004869   \n",
       "2019               0.068576             0.047258               0.048424   \n",
       "2020               0.134123             0.074121               0.042626   \n",
       "\n",
       "                                                \n",
       "      Japan JGB - SPJGBTR   EM Govt - GBIEMCOR  \n",
       "     calendar_year_return calendar_year_return  \n",
       "Date                                            \n",
       "2002             0.051543                  NaN  \n",
       "2003            -0.041043                  NaN  \n",
       "2004             0.005642                  NaN  \n",
       "2005             0.006645                  NaN  \n",
       "2006            -0.013907                  NaN  \n",
       "2007             0.040500                  NaN  \n",
       "2008             0.035918                  NaN  \n",
       "2009             0.025105            -0.088812  \n",
       "2010             0.024459             0.154323  \n",
       "2011             0.027175            -0.125680  \n",
       "2012             0.018526            -0.179821  \n",
       "2013             0.004020            -0.070201  \n",
       "2014             0.040626            -0.063224  \n",
       "2015             0.014520             0.155773  \n",
       "2016             0.010483            -0.165256  \n",
       "2017            -0.013668             0.146637  \n",
       "2018             0.014964            -0.024602  \n",
       "2019            -0.003778            -0.063896  \n",
       "2020            -0.024609            -0.350187  "
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr th {\n",
       "        text-align: left;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th colspan=\"5\" halign=\"left\">Govt Bond</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>US Treasury - LUATTRUU</th>\n",
       "      <th>UK Gilt - JFBG3GUS</th>\n",
       "      <th>German Bund - CBKIG0FT</th>\n",
       "      <th>Japan JGB - SPJGBTR</th>\n",
       "      <th>EM Govt - GBIEMCOR</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>Annualized Returns</th>\n",
       "      <td>0.031570</td>\n",
       "      <td>0.035009</td>\n",
       "      <td>0.025165</td>\n",
       "      <td>0.011475</td>\n",
       "      <td>-0.067697</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Annualized Volatility</th>\n",
       "      <td>0.055405</td>\n",
       "      <td>0.073377</td>\n",
       "      <td>0.049919</td>\n",
       "      <td>0.043905</td>\n",
       "      <td>0.084537</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Sharpe Ratio (assuming $R_f$ = 0)</th>\n",
       "      <td>0.128530</td>\n",
       "      <td>0.180850</td>\n",
       "      <td>0.139315</td>\n",
       "      <td>0.113346</td>\n",
       "      <td>-0.135072</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                                               Govt Bond                     \\\n",
       "                                  US Treasury - LUATTRUU UK Gilt - JFBG3GUS   \n",
       "Annualized Returns                              0.031570           0.035009   \n",
       "Annualized Volatility                           0.055405           0.073377   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)               0.128530           0.180850   \n",
       "\n",
       "                                                                              \\\n",
       "                                  German Bund - CBKIG0FT Japan JGB - SPJGBTR   \n",
       "Annualized Returns                              0.025165            0.011475   \n",
       "Annualized Volatility                           0.049919            0.043905   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)               0.139315            0.113346   \n",
       "\n",
       "                                                      \n",
       "                                  EM Govt - GBIEMCOR  \n",
       "Annualized Returns                         -0.067697  \n",
       "Annualized Volatility                       0.084537  \n",
       "Sharpe Ratio (assuming $R_f$ = 0)          -0.135072  "
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "====================================================================================================\n"
     ]
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr th {\n",
       "        text-align: left;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr:last-of-type th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th colspan=\"5\" halign=\"left\">Currencies</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>EUR/USD - EURUSD</th>\n",
       "      <th>JPY/USD - JPYUSD</th>\n",
       "      <th>GBP/USD - GBPUSD</th>\n",
       "      <th>AUD/USD - AUDUSD</th>\n",
       "      <th>CNY/USD - CNYUSD</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Date</th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>2002</th>\n",
       "      <td>0.148162</td>\n",
       "      <td>-0.077826</td>\n",
       "      <td>0.150727</td>\n",
       "      <td>0.006712</td>\n",
       "      <td>-0.000331</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2003</th>\n",
       "      <td>0.200438</td>\n",
       "      <td>0.053686</td>\n",
       "      <td>0.109193</td>\n",
       "      <td>0.339031</td>\n",
       "      <td>-0.000083</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2004</th>\n",
       "      <td>0.076141</td>\n",
       "      <td>0.044709</td>\n",
       "      <td>0.074140</td>\n",
       "      <td>0.037633</td>\n",
       "      <td>-0.000166</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2005</th>\n",
       "      <td>-0.102000</td>\n",
       "      <td>-0.064387</td>\n",
       "      <td>-0.046275</td>\n",
       "      <td>-0.021218</td>\n",
       "      <td>-0.018382</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2006</th>\n",
       "      <td>-0.050245</td>\n",
       "      <td>-0.045898</td>\n",
       "      <td>-0.039035</td>\n",
       "      <td>-0.094061</td>\n",
       "      <td>0.033091</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2007</th>\n",
       "      <td>0.105479</td>\n",
       "      <td>0.003912</td>\n",
       "      <td>0.013376</td>\n",
       "      <td>0.109829</td>\n",
       "      <td>0.069609</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2008</th>\n",
       "      <td>-0.044681</td>\n",
       "      <td>0.231672</td>\n",
       "      <td>0.347671</td>\n",
       "      <td>-0.009721</td>\n",
       "      <td>0.070411</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2009</th>\n",
       "      <td>-0.080276</td>\n",
       "      <td>-0.024317</td>\n",
       "      <td>-0.139300</td>\n",
       "      <td>-0.216322</td>\n",
       "      <td>-0.000819</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2010</th>\n",
       "      <td>-0.223406</td>\n",
       "      <td>0.046534</td>\n",
       "      <td>-0.167473</td>\n",
       "      <td>0.139913</td>\n",
       "      <td>0.034906</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2011</th>\n",
       "      <td>-0.100326</td>\n",
       "      <td>0.055019</td>\n",
       "      <td>-0.113537</td>\n",
       "      <td>-0.166821</td>\n",
       "      <td>0.047657</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2012</th>\n",
       "      <td>-0.025019</td>\n",
       "      <td>0.088735</td>\n",
       "      <td>-0.037251</td>\n",
       "      <td>-0.068677</td>\n",
       "      <td>0.011026</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2013</th>\n",
       "      <td>0.049704</td>\n",
       "      <td>0.204304</td>\n",
       "      <td>-0.144734</td>\n",
       "      <td>0.146359</td>\n",
       "      <td>0.029040</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2014</th>\n",
       "      <td>-0.012627</td>\n",
       "      <td>0.127919</td>\n",
       "      <td>-0.008513</td>\n",
       "      <td>-0.066779</td>\n",
       "      <td>-0.028018</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2015</th>\n",
       "      <td>0.100158</td>\n",
       "      <td>0.000587</td>\n",
       "      <td>0.049289</td>\n",
       "      <td>0.110821</td>\n",
       "      <td>0.042214</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2016</th>\n",
       "      <td>-0.078979</td>\n",
       "      <td>-0.102517</td>\n",
       "      <td>0.177952</td>\n",
       "      <td>-0.111267</td>\n",
       "      <td>0.068763</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2017</th>\n",
       "      <td>-0.004080</td>\n",
       "      <td>0.030011</td>\n",
       "      <td>-0.074729</td>\n",
       "      <td>0.028235</td>\n",
       "      <td>-0.039351</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2018</th>\n",
       "      <td>-0.007993</td>\n",
       "      <td>-0.023951</td>\n",
       "      <td>0.009791</td>\n",
       "      <td>-0.007616</td>\n",
       "      <td>-0.034823</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2019</th>\n",
       "      <td>0.020213</td>\n",
       "      <td>-0.047243</td>\n",
       "      <td>0.002487</td>\n",
       "      <td>-0.001233</td>\n",
       "      <td>0.009126</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2020</th>\n",
       "      <td>-0.056399</td>\n",
       "      <td>-0.020593</td>\n",
       "      <td>-0.053442</td>\n",
       "      <td>0.002394</td>\n",
       "      <td>-0.026229</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "               Currencies                                            \\\n",
       "         EUR/USD - EURUSD     JPY/USD - JPYUSD     GBP/USD - GBPUSD   \n",
       "     calendar_year_return calendar_year_return calendar_year_return   \n",
       "Date                                                                  \n",
       "2002             0.148162            -0.077826             0.150727   \n",
       "2003             0.200438             0.053686             0.109193   \n",
       "2004             0.076141             0.044709             0.074140   \n",
       "2005            -0.102000            -0.064387            -0.046275   \n",
       "2006            -0.050245            -0.045898            -0.039035   \n",
       "2007             0.105479             0.003912             0.013376   \n",
       "2008            -0.044681             0.231672             0.347671   \n",
       "2009            -0.080276            -0.024317            -0.139300   \n",
       "2010            -0.223406             0.046534            -0.167473   \n",
       "2011            -0.100326             0.055019            -0.113537   \n",
       "2012            -0.025019             0.088735            -0.037251   \n",
       "2013             0.049704             0.204304            -0.144734   \n",
       "2014            -0.012627             0.127919            -0.008513   \n",
       "2015             0.100158             0.000587             0.049289   \n",
       "2016            -0.078979            -0.102517             0.177952   \n",
       "2017            -0.004080             0.030011            -0.074729   \n",
       "2018            -0.007993            -0.023951             0.009791   \n",
       "2019             0.020213            -0.047243             0.002487   \n",
       "2020            -0.056399            -0.020593            -0.053442   \n",
       "\n",
       "                                                \n",
       "         AUD/USD - AUDUSD     CNY/USD - CNYUSD  \n",
       "     calendar_year_return calendar_year_return  \n",
       "Date                                            \n",
       "2002             0.006712            -0.000331  \n",
       "2003             0.339031            -0.000083  \n",
       "2004             0.037633            -0.000166  \n",
       "2005            -0.021218            -0.018382  \n",
       "2006            -0.094061             0.033091  \n",
       "2007             0.109829             0.069609  \n",
       "2008            -0.009721             0.070411  \n",
       "2009            -0.216322            -0.000819  \n",
       "2010             0.139913             0.034906  \n",
       "2011            -0.166821             0.047657  \n",
       "2012            -0.068677             0.011026  \n",
       "2013             0.146359             0.029040  \n",
       "2014            -0.066779            -0.028018  \n",
       "2015             0.110821             0.042214  \n",
       "2016            -0.111267             0.068763  \n",
       "2017             0.028235            -0.039351  \n",
       "2018            -0.007616            -0.034823  \n",
       "2019            -0.001233             0.009126  \n",
       "2020             0.002394            -0.026229  "
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr th {\n",
       "        text-align: left;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th colspan=\"5\" halign=\"left\">Currencies</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>EUR/USD - EURUSD</th>\n",
       "      <th>JPY/USD - JPYUSD</th>\n",
       "      <th>GBP/USD - GBPUSD</th>\n",
       "      <th>AUD/USD - AUDUSD</th>\n",
       "      <th>CNY/USD - CNYUSD</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>Annualized Returns</th>\n",
       "      <td>-0.009381</td>\n",
       "      <td>0.021707</td>\n",
       "      <td>-0.001322</td>\n",
       "      <td>0.001016</td>\n",
       "      <td>0.013498</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Annualized Volatility</th>\n",
       "      <td>0.136584</td>\n",
       "      <td>0.133411</td>\n",
       "      <td>0.074319</td>\n",
       "      <td>0.168132</td>\n",
       "      <td>0.073629</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Sharpe Ratio (assuming $R_f$ = 0)</th>\n",
       "      <td>-0.021918</td>\n",
       "      <td>0.056978</td>\n",
       "      <td>-0.002456</td>\n",
       "      <td>0.001898</td>\n",
       "      <td>0.089492</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                                        Currencies                   \\\n",
       "                                  EUR/USD - EURUSD JPY/USD - JPYUSD   \n",
       "Annualized Returns                       -0.009381         0.021707   \n",
       "Annualized Volatility                     0.136584         0.133411   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)        -0.021918         0.056978   \n",
       "\n",
       "                                                                     \\\n",
       "                                  GBP/USD - GBPUSD AUD/USD - AUDUSD   \n",
       "Annualized Returns                       -0.001322         0.001016   \n",
       "Annualized Volatility                     0.074319         0.168132   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)        -0.002456         0.001898   \n",
       "\n",
       "                                                    \n",
       "                                  CNY/USD - CNYUSD  \n",
       "Annualized Returns                        0.013498  \n",
       "Annualized Volatility                     0.073629  \n",
       "Sharpe Ratio (assuming $R_f$ = 0)         0.089492  "
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "====================================================================================================\n"
     ]
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr th {\n",
       "        text-align: left;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr:last-of-type th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th colspan=\"6\" halign=\"left\">Commod</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>WTI Oil - CL1 COMB</th>\n",
       "      <th>Brent Oil - CO1</th>\n",
       "      <th>Gold - XAUUSD</th>\n",
       "      <th>Silver - XAG</th>\n",
       "      <th>Copper - SPGSIC</th>\n",
       "      <th>Agriculture - BCOMAGTR</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Date</th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>2002</th>\n",
       "      <td>-0.137678</td>\n",
       "      <td>-0.142562</td>\n",
       "      <td>0.247715</td>\n",
       "      <td>-0.073021</td>\n",
       "      <td>-0.196890</td>\n",
       "      <td>0.041818</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2003</th>\n",
       "      <td>-0.218048</td>\n",
       "      <td>-0.163959</td>\n",
       "      <td>0.193650</td>\n",
       "      <td>-0.131105</td>\n",
       "      <td>0.081011</td>\n",
       "      <td>-0.015321</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2004</th>\n",
       "      <td>0.131023</td>\n",
       "      <td>0.136938</td>\n",
       "      <td>0.055362</td>\n",
       "      <td>0.148569</td>\n",
       "      <td>0.389583</td>\n",
       "      <td>-0.165351</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2005</th>\n",
       "      <td>0.404833</td>\n",
       "      <td>0.457736</td>\n",
       "      <td>0.179154</td>\n",
       "      <td>0.370638</td>\n",
       "      <td>0.401533</td>\n",
       "      <td>-0.238630</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2006</th>\n",
       "      <td>-0.016730</td>\n",
       "      <td>0.152491</td>\n",
       "      <td>0.231528</td>\n",
       "      <td>0.463982</td>\n",
       "      <td>0.408749</td>\n",
       "      <td>0.108420</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2007</th>\n",
       "      <td>-0.038936</td>\n",
       "      <td>0.007421</td>\n",
       "      <td>0.309408</td>\n",
       "      <td>-0.137633</td>\n",
       "      <td>-0.053572</td>\n",
       "      <td>0.298885</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2008</th>\n",
       "      <td>-0.001325</td>\n",
       "      <td>-0.056533</td>\n",
       "      <td>-0.185666</td>\n",
       "      <td>-0.181195</td>\n",
       "      <td>0.671353</td>\n",
       "      <td>-0.012467</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2009</th>\n",
       "      <td>-0.473938</td>\n",
       "      <td>-0.434962</td>\n",
       "      <td>0.305817</td>\n",
       "      <td>-0.231319</td>\n",
       "      <td>-0.515501</td>\n",
       "      <td>-0.009318</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2010</th>\n",
       "      <td>0.151462</td>\n",
       "      <td>0.215835</td>\n",
       "      <td>0.295739</td>\n",
       "      <td>0.831556</td>\n",
       "      <td>0.308329</td>\n",
       "      <td>0.003032</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2011</th>\n",
       "      <td>-0.243198</td>\n",
       "      <td>0.133298</td>\n",
       "      <td>0.100964</td>\n",
       "      <td>-0.099410</td>\n",
       "      <td>-0.342450</td>\n",
       "      <td>-0.105827</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2012</th>\n",
       "      <td>-0.105795</td>\n",
       "      <td>0.074718</td>\n",
       "      <td>-0.073634</td>\n",
       "      <td>-0.421993</td>\n",
       "      <td>-0.183567</td>\n",
       "      <td>-0.257674</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2013</th>\n",
       "      <td>0.055780</td>\n",
       "      <td>0.264209</td>\n",
       "      <td>0.321928</td>\n",
       "      <td>0.517521</td>\n",
       "      <td>0.116259</td>\n",
       "      <td>0.001788</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2014</th>\n",
       "      <td>0.600033</td>\n",
       "      <td>0.705686</td>\n",
       "      <td>-0.068142</td>\n",
       "      <td>0.007230</td>\n",
       "      <td>0.138502</td>\n",
       "      <td>-0.026743</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2015</th>\n",
       "      <td>0.230527</td>\n",
       "      <td>0.279488</td>\n",
       "      <td>-0.021973</td>\n",
       "      <td>0.095398</td>\n",
       "      <td>0.278538</td>\n",
       "      <td>0.138459</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2016</th>\n",
       "      <td>-0.229107</td>\n",
       "      <td>-0.271858</td>\n",
       "      <td>-0.217889</td>\n",
       "      <td>-0.522722</td>\n",
       "      <td>-0.275194</td>\n",
       "      <td>0.011708</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2017</th>\n",
       "      <td>0.032603</td>\n",
       "      <td>-0.034583</td>\n",
       "      <td>0.006521</td>\n",
       "      <td>0.005963</td>\n",
       "      <td>0.308171</td>\n",
       "      <td>0.051301</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2018</th>\n",
       "      <td>-0.065707</td>\n",
       "      <td>-0.048602</td>\n",
       "      <td>-0.106186</td>\n",
       "      <td>0.126177</td>\n",
       "      <td>-0.091015</td>\n",
       "      <td>0.098412</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2019</th>\n",
       "      <td>-0.163310</td>\n",
       "      <td>-0.072973</td>\n",
       "      <td>0.139826</td>\n",
       "      <td>0.035179</td>\n",
       "      <td>-0.050574</td>\n",
       "      <td>-0.036729</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2020</th>\n",
       "      <td>-0.940855</td>\n",
       "      <td>-0.506588</td>\n",
       "      <td>0.476989</td>\n",
       "      <td>0.597779</td>\n",
       "      <td>-0.154135</td>\n",
       "      <td>-0.062749</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                   Commod                                            \\\n",
       "       WTI Oil - CL1 COMB      Brent Oil - CO1        Gold - XAUUSD   \n",
       "     calendar_year_return calendar_year_return calendar_year_return   \n",
       "Date                                                                  \n",
       "2002            -0.137678            -0.142562             0.247715   \n",
       "2003            -0.218048            -0.163959             0.193650   \n",
       "2004             0.131023             0.136938             0.055362   \n",
       "2005             0.404833             0.457736             0.179154   \n",
       "2006            -0.016730             0.152491             0.231528   \n",
       "2007            -0.038936             0.007421             0.309408   \n",
       "2008            -0.001325            -0.056533            -0.185666   \n",
       "2009            -0.473938            -0.434962             0.305817   \n",
       "2010             0.151462             0.215835             0.295739   \n",
       "2011            -0.243198             0.133298             0.100964   \n",
       "2012            -0.105795             0.074718            -0.073634   \n",
       "2013             0.055780             0.264209             0.321928   \n",
       "2014             0.600033             0.705686            -0.068142   \n",
       "2015             0.230527             0.279488            -0.021973   \n",
       "2016            -0.229107            -0.271858            -0.217889   \n",
       "2017             0.032603            -0.034583             0.006521   \n",
       "2018            -0.065707            -0.048602            -0.106186   \n",
       "2019            -0.163310            -0.072973             0.139826   \n",
       "2020            -0.940855            -0.506588             0.476989   \n",
       "\n",
       "                                                                       \n",
       "             Silver - XAG      Copper - SPGSIC Agriculture - BCOMAGTR  \n",
       "     calendar_year_return calendar_year_return   calendar_year_return  \n",
       "Date                                                                   \n",
       "2002            -0.073021            -0.196890               0.041818  \n",
       "2003            -0.131105             0.081011              -0.015321  \n",
       "2004             0.148569             0.389583              -0.165351  \n",
       "2005             0.370638             0.401533              -0.238630  \n",
       "2006             0.463982             0.408749               0.108420  \n",
       "2007            -0.137633            -0.053572               0.298885  \n",
       "2008            -0.181195             0.671353              -0.012467  \n",
       "2009            -0.231319            -0.515501              -0.009318  \n",
       "2010             0.831556             0.308329               0.003032  \n",
       "2011            -0.099410            -0.342450              -0.105827  \n",
       "2012            -0.421993            -0.183567              -0.257674  \n",
       "2013             0.517521             0.116259               0.001788  \n",
       "2014             0.007230             0.138502              -0.026743  \n",
       "2015             0.095398             0.278538               0.138459  \n",
       "2016            -0.522722            -0.275194               0.011708  \n",
       "2017             0.005963             0.308171               0.051301  \n",
       "2018             0.126177            -0.091015               0.098412  \n",
       "2019             0.035179            -0.050574              -0.036729  \n",
       "2020             0.597779            -0.154135              -0.062749  "
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr th {\n",
       "        text-align: left;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th colspan=\"6\" halign=\"left\">Commod</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>WTI Oil - CL1 COMB</th>\n",
       "      <th>Brent Oil - CO1</th>\n",
       "      <th>Gold - XAUUSD</th>\n",
       "      <th>Silver - XAG</th>\n",
       "      <th>Copper - SPGSIC</th>\n",
       "      <th>Agriculture - BCOMAGTR</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>Annualized Returns</th>\n",
       "      <td>-0.166098</td>\n",
       "      <td>-0.004430</td>\n",
       "      <td>0.098833</td>\n",
       "      <td>0.019317</td>\n",
       "      <td>0.019451</td>\n",
       "      <td>-0.017449</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Annualized Volatility</th>\n",
       "      <td>0.794941</td>\n",
       "      <td>0.562311</td>\n",
       "      <td>0.221906</td>\n",
       "      <td>0.509008</td>\n",
       "      <td>0.387265</td>\n",
       "      <td>0.309227</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Sharpe Ratio (assuming $R_f$ = 0)</th>\n",
       "      <td>-0.120575</td>\n",
       "      <td>-0.003592</td>\n",
       "      <td>0.120173</td>\n",
       "      <td>0.013014</td>\n",
       "      <td>0.014807</td>\n",
       "      <td>-0.031794</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                                              Commod                  \\\n",
       "                                  WTI Oil - CL1 COMB Brent Oil - CO1   \n",
       "Annualized Returns                         -0.166098       -0.004430   \n",
       "Annualized Volatility                       0.794941        0.562311   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)          -0.120575       -0.003592   \n",
       "\n",
       "                                                                              \\\n",
       "                                  Gold - XAUUSD Silver - XAG Copper - SPGSIC   \n",
       "Annualized Returns                     0.098833     0.019317        0.019451   \n",
       "Annualized Volatility                  0.221906     0.509008        0.387265   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)      0.120173     0.013014        0.014807   \n",
       "\n",
       "                                                          \n",
       "                                  Agriculture - BCOMAGTR  \n",
       "Annualized Returns                             -0.017449  \n",
       "Annualized Volatility                           0.309227  \n",
       "Sharpe Ratio (assuming $R_f$ = 0)              -0.031794  "
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "====================================================================================================\n"
     ]
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr th {\n",
       "        text-align: left;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr:last-of-type th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th colspan=\"5\" halign=\"left\">EQ Factors</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>Value - DJTMNSV</th>\n",
       "      <th>Size - DJTMNSS</th>\n",
       "      <th>Quality - DJTMNQU</th>\n",
       "      <th>Momentum - DJTMNMO</th>\n",
       "      <th>Low Beta - DJTMNAB</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "      <th>calendar_year_return</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Date</th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "      <th></th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>2002</th>\n",
       "      <td>NaN</td>\n",
       "      <td>NaN</td>\n",
       "      <td>NaN</td>\n",
       "      <td>NaN</td>\n",
       "      <td>NaN</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2003</th>\n",
       "      <td>0.128332</td>\n",
       "      <td>-0.045934</td>\n",
       "      <td>-0.027136</td>\n",
       "      <td>-0.114272</td>\n",
       "      <td>-0.095846</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2004</th>\n",
       "      <td>0.124360</td>\n",
       "      <td>0.089198</td>\n",
       "      <td>-0.016135</td>\n",
       "      <td>-0.021923</td>\n",
       "      <td>-0.020721</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2005</th>\n",
       "      <td>0.015967</td>\n",
       "      <td>0.022662</td>\n",
       "      <td>0.042710</td>\n",
       "      <td>0.151704</td>\n",
       "      <td>-0.007309</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2006</th>\n",
       "      <td>-0.067420</td>\n",
       "      <td>0.045008</td>\n",
       "      <td>0.006538</td>\n",
       "      <td>-0.000067</td>\n",
       "      <td>-0.155052</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2007</th>\n",
       "      <td>0.064438</td>\n",
       "      <td>0.014825</td>\n",
       "      <td>0.073109</td>\n",
       "      <td>0.202739</td>\n",
       "      <td>-0.000901</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2008</th>\n",
       "      <td>-0.084118</td>\n",
       "      <td>0.002846</td>\n",
       "      <td>0.159003</td>\n",
       "      <td>0.080683</td>\n",
       "      <td>0.351417</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2009</th>\n",
       "      <td>0.339859</td>\n",
       "      <td>0.151237</td>\n",
       "      <td>-0.060329</td>\n",
       "      <td>-0.092819</td>\n",
       "      <td>-0.164237</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2010</th>\n",
       "      <td>-0.066073</td>\n",
       "      <td>0.119517</td>\n",
       "      <td>0.051802</td>\n",
       "      <td>-0.000386</td>\n",
       "      <td>0.153621</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2011</th>\n",
       "      <td>0.032565</td>\n",
       "      <td>0.015253</td>\n",
       "      <td>0.089126</td>\n",
       "      <td>0.067082</td>\n",
       "      <td>-0.299770</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2012</th>\n",
       "      <td>0.083043</td>\n",
       "      <td>0.032431</td>\n",
       "      <td>0.037111</td>\n",
       "      <td>0.076830</td>\n",
       "      <td>-0.060622</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2013</th>\n",
       "      <td>0.158641</td>\n",
       "      <td>0.063753</td>\n",
       "      <td>-0.003690</td>\n",
       "      <td>0.003273</td>\n",
       "      <td>0.030478</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2014</th>\n",
       "      <td>0.024173</td>\n",
       "      <td>-0.068263</td>\n",
       "      <td>-0.110608</td>\n",
       "      <td>-0.003507</td>\n",
       "      <td>-0.013651</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2015</th>\n",
       "      <td>-0.022226</td>\n",
       "      <td>0.051858</td>\n",
       "      <td>0.103951</td>\n",
       "      <td>0.013864</td>\n",
       "      <td>0.083934</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2016</th>\n",
       "      <td>0.077949</td>\n",
       "      <td>-0.059151</td>\n",
       "      <td>-0.121989</td>\n",
       "      <td>-0.008552</td>\n",
       "      <td>-0.018173</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2017</th>\n",
       "      <td>-0.087912</td>\n",
       "      <td>-0.076334</td>\n",
       "      <td>0.100932</td>\n",
       "      <td>-0.038273</td>\n",
       "      <td>0.007228</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2018</th>\n",
       "      <td>0.140675</td>\n",
       "      <td>0.048635</td>\n",
       "      <td>0.044165</td>\n",
       "      <td>0.057604</td>\n",
       "      <td>0.050381</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2019</th>\n",
       "      <td>-0.012805</td>\n",
       "      <td>-0.017005</td>\n",
       "      <td>0.004608</td>\n",
       "      <td>-0.199652</td>\n",
       "      <td>0.030855</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2020</th>\n",
       "      <td>0.712308</td>\n",
       "      <td>0.276804</td>\n",
       "      <td>-0.165403</td>\n",
       "      <td>0.132956</td>\n",
       "      <td>0.142463</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "              EQ Factors                                             \\\n",
       "          Value - DJTMNSV       Size - DJTMNSS    Quality - DJTMNQU   \n",
       "     calendar_year_return calendar_year_return calendar_year_return   \n",
       "Date                                                                  \n",
       "2002                  NaN                  NaN                  NaN   \n",
       "2003             0.128332            -0.045934            -0.027136   \n",
       "2004             0.124360             0.089198            -0.016135   \n",
       "2005             0.015967             0.022662             0.042710   \n",
       "2006            -0.067420             0.045008             0.006538   \n",
       "2007             0.064438             0.014825             0.073109   \n",
       "2008            -0.084118             0.002846             0.159003   \n",
       "2009             0.339859             0.151237            -0.060329   \n",
       "2010            -0.066073             0.119517             0.051802   \n",
       "2011             0.032565             0.015253             0.089126   \n",
       "2012             0.083043             0.032431             0.037111   \n",
       "2013             0.158641             0.063753            -0.003690   \n",
       "2014             0.024173            -0.068263            -0.110608   \n",
       "2015            -0.022226             0.051858             0.103951   \n",
       "2016             0.077949            -0.059151            -0.121989   \n",
       "2017            -0.087912            -0.076334             0.100932   \n",
       "2018             0.140675             0.048635             0.044165   \n",
       "2019            -0.012805            -0.017005             0.004608   \n",
       "2020             0.712308             0.276804            -0.165403   \n",
       "\n",
       "                                                \n",
       "       Momentum - DJTMNMO   Low Beta - DJTMNAB  \n",
       "     calendar_year_return calendar_year_return  \n",
       "Date                                            \n",
       "2002                  NaN                  NaN  \n",
       "2003            -0.114272            -0.095846  \n",
       "2004            -0.021923            -0.020721  \n",
       "2005             0.151704            -0.007309  \n",
       "2006            -0.000067            -0.155052  \n",
       "2007             0.202739            -0.000901  \n",
       "2008             0.080683             0.351417  \n",
       "2009            -0.092819            -0.164237  \n",
       "2010            -0.000386             0.153621  \n",
       "2011             0.067082            -0.299770  \n",
       "2012             0.076830            -0.060622  \n",
       "2013             0.003273             0.030478  \n",
       "2014            -0.003507            -0.013651  \n",
       "2015             0.013864             0.083934  \n",
       "2016            -0.008552            -0.018173  \n",
       "2017            -0.038273             0.007228  \n",
       "2018             0.057604             0.050381  \n",
       "2019            -0.199652             0.030855  \n",
       "2020             0.132956             0.142463  "
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead tr th {\n",
       "        text-align: left;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th colspan=\"5\" halign=\"left\">EQ Factors</th>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th></th>\n",
       "      <th>Value - DJTMNSV</th>\n",
       "      <th>Size - DJTMNSS</th>\n",
       "      <th>Quality - DJTMNQU</th>\n",
       "      <th>Momentum - DJTMNMO</th>\n",
       "      <th>Low Beta - DJTMNAB</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>Annualized Returns</th>\n",
       "      <td>0.073872</td>\n",
       "      <td>0.033848</td>\n",
       "      <td>0.008022</td>\n",
       "      <td>0.012581</td>\n",
       "      <td>-0.008529</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Annualized Volatility</th>\n",
       "      <td>0.146757</td>\n",
       "      <td>0.111505</td>\n",
       "      <td>0.153330</td>\n",
       "      <td>0.291844</td>\n",
       "      <td>0.232533</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Sharpe Ratio (assuming $R_f$ = 0)</th>\n",
       "      <td>0.094764</td>\n",
       "      <td>0.095148</td>\n",
       "      <td>0.022765</td>\n",
       "      <td>0.031399</td>\n",
       "      <td>-0.014766</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                                      EQ Factors                  \\\n",
       "                                  Value - DJTMNSV Size - DJTMNSS   \n",
       "Annualized Returns                       0.073872       0.033848   \n",
       "Annualized Volatility                    0.146757       0.111505   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)        0.094764       0.095148   \n",
       "\n",
       "                                                                        \\\n",
       "                                  Quality - DJTMNQU Momentum - DJTMNMO   \n",
       "Annualized Returns                         0.008022           0.012581   \n",
       "Annualized Volatility                      0.153330           0.291844   \n",
       "Sharpe Ratio (assuming $R_f$ = 0)          0.022765           0.031399   \n",
       "\n",
       "                                                      \n",
       "                                  Low Beta - DJTMNAB  \n",
       "Annualized Returns                         -0.008529  \n",
       "Annualized Volatility                       0.232533  \n",
       "Sharpe Ratio (assuming $R_f$ = 0)          -0.014766  "
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "application/javascript": [
       "\n",
       "            setTimeout(function() {\n",
       "                var nbb_cell_id = 4;\n",
       "                var nbb_unformatted_code = \"for asset_class in data.columns.get_level_values(\\\"asset_class\\\").unique():\\n    print(\\\"=\\\" * 100)\\n    tsms = TimeSeriesMomentumStrategy(asset_class=asset_class, asset_class_history=data)\\n    tsms.execute_backtest()\";\n",
       "                var nbb_formatted_code = \"for asset_class in data.columns.get_level_values(\\\"asset_class\\\").unique():\\n    print(\\\"=\\\" * 100)\\n    tsms = TimeSeriesMomentumStrategy(asset_class=asset_class, asset_class_history=data)\\n    tsms.execute_backtest()\";\n",
       "                var nbb_cells = Jupyter.notebook.get_cells();\n",
       "                for (var i = 0; i < nbb_cells.length; ++i) {\n",
       "                    if (nbb_cells[i].input_prompt_number == nbb_cell_id) {\n",
       "                        if (nbb_cells[i].get_text() == nbb_unformatted_code) {\n",
       "                             nbb_cells[i].set_text(nbb_formatted_code);\n",
       "                        }\n",
       "                        break;\n",
       "                    }\n",
       "                }\n",
       "            }, 500);\n",
       "            "
      ],
      "text/plain": [
       "<IPython.core.display.Javascript object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "time: 1.35 s\n"
     ]
    }
   ],
   "source": [
    "for asset_class in data.columns.get_level_values(\"asset_class\").unique():\n",
    "    print(\"=\" * 100)\n",
    "    tsms = TimeSeriesMomentumStrategy(asset_class=asset_class, asset_class_history=data)\n",
    "    tsms.execute_backtest()"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Cross-sectional Momentum Strategy"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "application/javascript": [
       "\n",
       "            setTimeout(function() {\n",
       "                var nbb_cell_id = 10;\n",
       "                var nbb_unformatted_code = \"class CrossSectionalMomentumStrategy(TimeSeriesMomentumStrategy):\\n    \\\"\\\"\\\"Longs 50% of best performing asset in past 12 months in asset class \\n    and Shorts 50% of worst performing asset in past 12 months in asset class for 1 month  \\n    Assumes that all transactions happen on the end of month\\n    \\\"\\\"\\\"\\n\\n    def __init__(self, momentum_signal_period: int = 12, **kwargs):\\n        TradingStrategy.__init__(\\n            self, name=\\\"Cross Sectional Momentum Strategy\\\", **kwargs\\n        )\\n        self.momentum_signal_period = momentum_signal_period\\n        self.target_asset_history = self.asset_class_history[\\n            self.asset_class_history.columns[\\n                self.asset_class_history.columns.get_level_values(\\\"asset_class\\\")\\n                == asset_class\\n            ]\\n        ]\\n\\n    def execute_backtest(\\n        self,\\n        start_date: str = \\\"2001-12-31\\\",\\n        end_date: str = \\\"2020-08-31\\\",\\n        date_format: str = \\\"%Y-%m-%d\\\",\\n    ):\\n        \\\"\\\"\\\"Contains the actual trading logic \\\"\\\"\\\"\\n        self.start_date = start_date\\n        self.end_date = end_date\\n        self.date_format = date_format\\n        self.only_month_end_df = self.filter_month_end(df=self.target_asset_history)\\n        self.check_data()\\n        self.past_12_month_returns = self.only_month_end_df.pct_change(\\n            self.momentum_signal_period\\n        ).iloc[\\n            self.start_date_idx : self.end_date_idx + 1\\n        ]  # Cumulative returns from past 12 months\\n        self.monthly_long_position_returns = (\\n            self.only_month_end_df.iloc[self.start_date_idx - 1 :]\\n            .pct_change(1)\\n            .iloc[1 : self.end_date_idx + 1]\\n        )  # End of Month Monthly Long position returns\\n\\n        long_portfolio_returns = (\\n            self.past_12_month_returns.apply(lambda row: (row == np.max(row)), axis=1)\\n            .shift(1)\\n            .iloc[1:]\\n            * self.monthly_long_position_returns.iloc[1:]\\n        ).sum(\\n            axis=1\\n        )  # Longs 50% of best performing asset in past 12 months in asset class\\n\\n        short_portfolio_returns = -(\\n            self.past_12_month_returns.apply(lambda row: (row == np.min(row)), axis=1)\\n            .shift(1)\\n            .iloc[1:]\\n            * self.monthly_long_position_returns.iloc[1:]\\n        ).sum(\\n            axis=1\\n        )  # Shorts 50% of worst performing asset in past 12 months in asset class\\n\\n        self.monthly_returns = (\\n            long_portfolio_returns * 0.5 + short_portfolio_returns * 0.5\\n        )  # Weight portfolios\\n        self._results()\\n\\n    def _results(self):\\n        \\\"\\\"\\\"\\\"\\\"\\\"\\n        self.annual_results = self.monthly_returns.groupby(\\n            [self.monthly_returns.index.year]\\n        ).agg([self.calendar_year_return])\\n        self.asset_class_annualized_returns = pd.Series(\\n            self.annual_results.apply(self.annualized_return, axis=0).values,\\n            name=\\\"Annualized Returns\\\",\\n        )\\n        self.asset_class_annualized_volatility = pd.Series(\\n            (\\n                self.monthly_returns.groupby([self.monthly_returns.index.year])\\n                .agg([self.annualized_volatility])\\n                .apply(self.annualized_volatility, axis=0)\\n            ).values,\\n            name=\\\"Annualized Volatility\\\",\\n        )\\n        self.asset_class_sharpe_ratio = pd.Series(\\n            self.annual_results.apply(self.sharpe_ratio, axis=0).values,\\n            name=\\\"Sharpe Ratio (assuming $R_f$ = 0)\\\",\\n        )\\n        self.annual_results = self.annual_results.style.set_caption(self.asset_class)\\n        display(self.annual_results)\\n        summary = pd.concat(\\n            [\\n                self.asset_class_annualized_returns,\\n                self.asset_class_annualized_volatility,\\n                self.asset_class_sharpe_ratio,\\n            ],\\n            axis=1,\\n        ).transpose()\\n        summary.columns = [\\\"Summary\\\"]\\n        display(summary)\";\n",
       "                var nbb_formatted_code = \"class CrossSectionalMomentumStrategy(TimeSeriesMomentumStrategy):\\n    \\\"\\\"\\\"Longs 50% of best performing asset in past 12 months in asset class \\n    and Shorts 50% of worst performing asset in past 12 months in asset class for 1 month  \\n    Assumes that all transactions happen on the end of month\\n    \\\"\\\"\\\"\\n\\n    def __init__(self, momentum_signal_period: int = 12, **kwargs):\\n        TradingStrategy.__init__(\\n            self, name=\\\"Cross Sectional Momentum Strategy\\\", **kwargs\\n        )\\n        self.momentum_signal_period = momentum_signal_period\\n        self.target_asset_history = self.asset_class_history[\\n            self.asset_class_history.columns[\\n                self.asset_class_history.columns.get_level_values(\\\"asset_class\\\")\\n                == asset_class\\n            ]\\n        ]\\n\\n    def execute_backtest(\\n        self,\\n        start_date: str = \\\"2001-12-31\\\",\\n        end_date: str = \\\"2020-08-31\\\",\\n        date_format: str = \\\"%Y-%m-%d\\\",\\n    ):\\n        \\\"\\\"\\\"Contains the actual trading logic \\\"\\\"\\\"\\n        self.start_date = start_date\\n        self.end_date = end_date\\n        self.date_format = date_format\\n        self.only_month_end_df = self.filter_month_end(df=self.target_asset_history)\\n        self.check_data()\\n        self.past_12_month_returns = self.only_month_end_df.pct_change(\\n            self.momentum_signal_period\\n        ).iloc[\\n            self.start_date_idx : self.end_date_idx + 1\\n        ]  # Cumulative returns from past 12 months\\n        self.monthly_long_position_returns = (\\n            self.only_month_end_df.iloc[self.start_date_idx - 1 :]\\n            .pct_change(1)\\n            .iloc[1 : self.end_date_idx + 1]\\n        )  # End of Month Monthly Long position returns\\n\\n        long_portfolio_returns = (\\n            self.past_12_month_returns.apply(lambda row: (row == np.max(row)), axis=1)\\n            .shift(1)\\n            .iloc[1:]\\n            * self.monthly_long_position_returns.iloc[1:]\\n        ).sum(\\n            axis=1\\n        )  # Longs 50% of best performing asset in past 12 months in asset class\\n\\n        short_portfolio_returns = -(\\n            self.past_12_month_returns.apply(lambda row: (row == np.min(row)), axis=1)\\n            .shift(1)\\n            .iloc[1:]\\n            * self.monthly_long_position_returns.iloc[1:]\\n        ).sum(\\n            axis=1\\n        )  # Shorts 50% of worst performing asset in past 12 months in asset class\\n\\n        self.monthly_returns = (\\n            long_portfolio_returns * 0.5 + short_portfolio_returns * 0.5\\n        )  # Weight portfolios\\n        self._results()\\n\\n    def _results(self):\\n        \\\"\\\"\\\"\\\"\\\"\\\"\\n        self.annual_results = self.monthly_returns.groupby(\\n            [self.monthly_returns.index.year]\\n        ).agg([self.calendar_year_return])\\n        self.asset_class_annualized_returns = pd.Series(\\n            self.annual_results.apply(self.annualized_return, axis=0).values,\\n            name=\\\"Annualized Returns\\\",\\n        )\\n        self.asset_class_annualized_volatility = pd.Series(\\n            (\\n                self.monthly_returns.groupby([self.monthly_returns.index.year])\\n                .agg([self.annualized_volatility])\\n                .apply(self.annualized_volatility, axis=0)\\n            ).values,\\n            name=\\\"Annualized Volatility\\\",\\n        )\\n        self.asset_class_sharpe_ratio = pd.Series(\\n            self.annual_results.apply(self.sharpe_ratio, axis=0).values,\\n            name=\\\"Sharpe Ratio (assuming $R_f$ = 0)\\\",\\n        )\\n        self.annual_results = self.annual_results.style.set_caption(self.asset_class)\\n        display(self.annual_results)\\n        summary = pd.concat(\\n            [\\n                self.asset_class_annualized_returns,\\n                self.asset_class_annualized_volatility,\\n                self.asset_class_sharpe_ratio,\\n            ],\\n            axis=1,\\n        ).transpose()\\n        summary.columns = [\\\"Summary\\\"]\\n        display(summary)\";\n",
       "                var nbb_cells = Jupyter.notebook.get_cells();\n",
       "                for (var i = 0; i < nbb_cells.length; ++i) {\n",
       "                    if (nbb_cells[i].input_prompt_number == nbb_cell_id) {\n",
       "                        if (nbb_cells[i].get_text() == nbb_unformatted_code) {\n",
       "                             nbb_cells[i].set_text(nbb_formatted_code);\n",
       "                        }\n",
       "                        break;\n",
       "                    }\n",
       "                }\n",
       "            }, 500);\n",
       "            "
      ],
      "text/plain": [
       "<IPython.core.display.Javascript object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "time: 63 ms\n"
     ]
    }
   ],
   "source": [
    "class CrossSectionalMomentumStrategy(TimeSeriesMomentumStrategy):\n",
    "    \"\"\"Longs 50% of best performing asset in past 12 months in asset class \n",
    "    and Shorts 50% of worst performing asset in past 12 months in asset class for 1 month  \n",
    "    Assumes that all transactions happen on the end of month\n",
    "    \"\"\"\n",
    "\n",
    "    def __init__(self, momentum_signal_period: int = 12, **kwargs):\n",
    "        TradingStrategy.__init__(\n",
    "            self, name=\"Cross Sectional Momentum Strategy\", **kwargs\n",
    "        )\n",
    "        self.momentum_signal_period = momentum_signal_period\n",
    "        self.target_asset_history = self.asset_class_history[\n",
    "            self.asset_class_history.columns[\n",
    "                self.asset_class_history.columns.get_level_values(\"asset_class\")\n",
    "                == asset_class\n",
    "            ]\n",
    "        ]\n",
    "\n",
    "    def execute_backtest(\n",
    "        self,\n",
    "        start_date: str = \"2001-12-31\",\n",
    "        end_date: str = \"2020-08-31\",\n",
    "        date_format: str = \"%Y-%m-%d\",\n",
    "    ):\n",
    "        \"\"\"Contains the actual trading logic \"\"\"\n",
    "        self.start_date = start_date\n",
    "        self.end_date = end_date\n",
    "        self.date_format = date_format\n",
    "        self.only_month_end_df = self.filter_month_end(df=self.target_asset_history)\n",
    "        self.check_data()\n",
    "        self.past_12_month_returns = self.only_month_end_df.pct_change(\n",
    "            self.momentum_signal_period\n",
    "        ).iloc[\n",
    "            self.start_date_idx : self.end_date_idx + 1\n",
    "        ]  # Cumulative returns from past 12 months\n",
    "        self.monthly_long_position_returns = (\n",
    "            self.only_month_end_df.iloc[self.start_date_idx - 1 :]\n",
    "            .pct_change(1)\n",
    "            .iloc[1 : self.end_date_idx + 1]\n",
    "        )  # End of Month Monthly Long position returns\n",
    "\n",
    "        long_portfolio_returns = (\n",
    "            self.past_12_month_returns.apply(lambda row: (row == np.max(row)), axis=1)\n",
    "            .shift(1)\n",
    "            .iloc[1:]\n",
    "            * self.monthly_long_position_returns.iloc[1:]\n",
    "        ).sum(\n",
    "            axis=1\n",
    "        )  # Longs 50% of best performing asset in past 12 months in asset class\n",
    "\n",
    "        short_portfolio_returns = -(\n",
    "            self.past_12_month_returns.apply(lambda row: (row == np.min(row)), axis=1)\n",
    "            .shift(1)\n",
    "            .iloc[1:]\n",
    "            * self.monthly_long_position_returns.iloc[1:]\n",
    "        ).sum(\n",
    "            axis=1\n",
    "        )  # Shorts 50% of worst performing asset in past 12 months in asset class\n",
    "\n",
    "        self.monthly_returns = (\n",
    "            long_portfolio_returns * 0.5 + short_portfolio_returns * 0.5\n",
    "        )  # Weight portfolios\n",
    "        self._results()\n",
    "\n",
    "    def _results(self):\n",
    "        \"\"\"\"\"\"\n",
    "        self.annual_results = self.monthly_returns.groupby(\n",
    "            [self.monthly_returns.index.year]\n",
    "        ).agg([self.calendar_year_return])\n",
    "        self.asset_class_annualized_returns = pd.Series(\n",
    "            self.annual_results.apply(self.annualized_return, axis=0).values,\n",
    "            name=\"Annualized Returns\",\n",
    "        )\n",
    "        self.asset_class_annualized_volatility = pd.Series(\n",
    "            (\n",
    "                self.monthly_returns.groupby([self.monthly_returns.index.year])\n",
    "                .agg([self.annualized_volatility])\n",
    "                .apply(self.annualized_volatility, axis=0)\n",
    "            ).values,\n",
    "            name=\"Annualized Volatility\",\n",
    "        )\n",
    "        self.asset_class_sharpe_ratio = pd.Series(\n",
    "            self.annual_results.apply(self.sharpe_ratio, axis=0).values,\n",
    "            name=\"Sharpe Ratio (assuming $R_f$ = 0)\",\n",
    "        )\n",
    "        self.annual_results = self.annual_results.style.set_caption(self.asset_class)\n",
    "        display(self.annual_results)\n",
    "        summary = pd.concat(\n",
    "            [\n",
    "                self.asset_class_annualized_returns,\n",
    "                self.asset_class_annualized_volatility,\n",
    "                self.asset_class_sharpe_ratio,\n",
    "            ],\n",
    "            axis=1,\n",
    "        ).transpose()\n",
    "        summary.columns = [\"Summary\"]\n",
    "        display(summary)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "metadata": {
    "scrolled": false
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "====================================================================================================\n"
     ]
    },
    {
     "data": {
      "text/html": [
       "<style  type=\"text/css\" >\n",
       "</style><table id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122\" ><caption>Equity Idx</caption><thead>    <tr>        <th class=\"blank level0\" ></th>        <th class=\"col_heading level0 col0\" >calendar_year_return</th>    </tr>    <tr>        <th class=\"index_name level0\" >Date</th>        <th class=\"blank\" ></th>    </tr></thead><tbody>\n",
       "                <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row0\" class=\"row_heading level0 row0\" >2002</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row0_col0\" class=\"data row0 col0\" >0.099955</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row1\" class=\"row_heading level0 row1\" >2003</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row1_col0\" class=\"data row1 col0\" >0.175910</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row2\" class=\"row_heading level0 row2\" >2004</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row2_col0\" class=\"data row2 col0\" >0.076786</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row3\" class=\"row_heading level0 row3\" >2005</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row3_col0\" class=\"data row3 col0\" >0.200651</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row4\" class=\"row_heading level0 row4\" >2006</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row4_col0\" class=\"data row4 col0\" >0.306280</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row5\" class=\"row_heading level0 row5\" >2007</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row5_col0\" class=\"data row5 col0\" >0.417425</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row6\" class=\"row_heading level0 row6\" >2008</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row6_col0\" class=\"data row6 col0\" >-0.139295</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row7\" class=\"row_heading level0 row7\" >2009</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row7_col0\" class=\"data row7 col0\" >-0.365068</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row8\" class=\"row_heading level0 row8\" >2010</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row8_col0\" class=\"data row8 col0\" >-0.063702</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row9\" class=\"row_heading level0 row9\" >2011</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row9_col0\" class=\"data row9 col0\" >-0.007779</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row10\" class=\"row_heading level0 row10\" >2012</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row10_col0\" class=\"data row10 col0\" >-0.043413</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row11\" class=\"row_heading level0 row11\" >2013</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row11_col0\" class=\"data row11 col0\" >0.113152</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row12\" class=\"row_heading level0 row12\" >2014</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row12_col0\" class=\"data row12 col0\" >-0.019550</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row13\" class=\"row_heading level0 row13\" >2015</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row13_col0\" class=\"data row13 col0\" >0.138860</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row14\" class=\"row_heading level0 row14\" >2016</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row14_col0\" class=\"data row14 col0\" >-0.247295</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row15\" class=\"row_heading level0 row15\" >2017</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row15_col0\" class=\"data row15 col0\" >0.008662</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row16\" class=\"row_heading level0 row16\" >2018</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row16_col0\" class=\"data row16 col0\" >-0.214263</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row17\" class=\"row_heading level0 row17\" >2019</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row17_col0\" class=\"data row17 col0\" >-0.030656</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122level0_row18\" class=\"row_heading level0 row18\" >2020</th>\n",
       "                        <td id=\"T_33ee80fc_f5a1_11ea_87db_acde48001122row18_col0\" class=\"data row18 col0\" >0.345217</td>\n",
       "            </tr>\n",
       "    </tbody></table>"
      ],
      "text/plain": [
       "<pandas.io.formats.style.Styler at 0x7fec4dc11110>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th></th>\n",
       "      <th>Summary</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>Annualized Returns</th>\n",
       "      <td>0.019892</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Annualized Volatility</th>\n",
       "      <td>0.163184</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Sharpe Ratio (assuming $R_f$ = 0)</th>\n",
       "      <td>0.023065</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                                    Summary\n",
       "Annualized Returns                 0.019892\n",
       "Annualized Volatility              0.163184\n",
       "Sharpe Ratio (assuming $R_f$ = 0)  0.023065"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "====================================================================================================\n"
     ]
    },
    {
     "data": {
      "text/html": [
       "<style  type=\"text/css\" >\n",
       "</style><table id=\"T_34072c42_f5a1_11ea_87db_acde48001122\" ><caption>Credit</caption><thead>    <tr>        <th class=\"blank level0\" ></th>        <th class=\"col_heading level0 col0\" >calendar_year_return</th>    </tr>    <tr>        <th class=\"index_name level0\" >Date</th>        <th class=\"blank\" ></th>    </tr></thead><tbody>\n",
       "                <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row0\" class=\"row_heading level0 row0\" >2002</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row0_col0\" class=\"data row0 col0\" >0.062447</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row1\" class=\"row_heading level0 row1\" >2003</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row1_col0\" class=\"data row1 col0\" >-0.020326</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row2\" class=\"row_heading level0 row2\" >2004</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row2_col0\" class=\"data row2 col0\" >0.034811</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row3\" class=\"row_heading level0 row3\" >2005</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row3_col0\" class=\"data row3 col0\" >0.003829</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row4\" class=\"row_heading level0 row4\" >2006</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row4_col0\" class=\"data row4 col0\" >0.036200</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row5\" class=\"row_heading level0 row5\" >2007</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row5_col0\" class=\"data row5 col0\" >0.013033</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row6\" class=\"row_heading level0 row6\" >2008</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row6_col0\" class=\"data row6 col0\" >0.114485</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row7\" class=\"row_heading level0 row7\" >2009</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row7_col0\" class=\"data row7 col0\" >-0.156851</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row8\" class=\"row_heading level0 row8\" >2010</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row8_col0\" class=\"data row8 col0\" >0.038716</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row9\" class=\"row_heading level0 row9\" >2011</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row9_col0\" class=\"data row9 col0\" >-0.028698</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row10\" class=\"row_heading level0 row10\" >2012</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row10_col0\" class=\"data row10 col0\" >-0.006391</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row11\" class=\"row_heading level0 row11\" >2013</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row11_col0\" class=\"data row11 col0\" >0.057092</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row12\" class=\"row_heading level0 row12\" >2014</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row12_col0\" class=\"data row12 col0\" >-0.000328</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row13\" class=\"row_heading level0 row13\" >2015</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row13_col0\" class=\"data row13 col0\" >-0.002702</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row14\" class=\"row_heading level0 row14\" >2016</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row14_col0\" class=\"data row14 col0\" >0.027312</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row15\" class=\"row_heading level0 row15\" >2017</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row15_col0\" class=\"data row15 col0\" >-0.016831</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row16\" class=\"row_heading level0 row16\" >2018</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row16_col0\" class=\"data row16 col0\" >-0.009025</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row17\" class=\"row_heading level0 row17\" >2019</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row17_col0\" class=\"data row17 col0\" >0.041674</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34072c42_f5a1_11ea_87db_acde48001122level0_row18\" class=\"row_heading level0 row18\" >2020</th>\n",
       "                        <td id=\"T_34072c42_f5a1_11ea_87db_acde48001122row18_col0\" class=\"data row18 col0\" >0.004932</td>\n",
       "            </tr>\n",
       "    </tbody></table>"
      ],
      "text/plain": [
       "<pandas.io.formats.style.Styler at 0x7fec4dc11b50>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th></th>\n",
       "      <th>Summary</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>Annualized Returns</th>\n",
       "      <td>0.008773</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Annualized Volatility</th>\n",
       "      <td>0.074868</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Sharpe Ratio (assuming $R_f$ = 0)</th>\n",
       "      <td>0.038750</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                                    Summary\n",
       "Annualized Returns                 0.008773\n",
       "Annualized Volatility              0.074868\n",
       "Sharpe Ratio (assuming $R_f$ = 0)  0.038750"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "====================================================================================================\n"
     ]
    },
    {
     "data": {
      "text/html": [
       "<style  type=\"text/css\" >\n",
       "</style><table id=\"T_341fe692_f5a1_11ea_87db_acde48001122\" ><caption>Govt Bond</caption><thead>    <tr>        <th class=\"blank level0\" ></th>        <th class=\"col_heading level0 col0\" >calendar_year_return</th>    </tr>    <tr>        <th class=\"index_name level0\" >Date</th>        <th class=\"blank\" ></th>    </tr></thead><tbody>\n",
       "                <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row0\" class=\"row_heading level0 row0\" >2002</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row0_col0\" class=\"data row0 col0\" >0.014456</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row1\" class=\"row_heading level0 row1\" >2003</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row1_col0\" class=\"data row1 col0\" >-0.020465</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row2\" class=\"row_heading level0 row2\" >2004</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row2_col0\" class=\"data row2 col0\" >0.000196</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row3\" class=\"row_heading level0 row3\" >2005</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row3_col0\" class=\"data row3 col0\" >0.008041</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row4\" class=\"row_heading level0 row4\" >2006</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row4_col0\" class=\"data row4 col0\" >0.013660</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row5\" class=\"row_heading level0 row5\" >2007</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row5_col0\" class=\"data row5 col0\" >0.053238</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row6\" class=\"row_heading level0 row6\" >2008</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row6_col0\" class=\"data row6 col0\" >0.012984</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row7\" class=\"row_heading level0 row7\" >2009</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row7_col0\" class=\"data row7 col0\" >-0.079125</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row8\" class=\"row_heading level0 row8\" >2010</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row8_col0\" class=\"data row8 col0\" >0.031045</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row9\" class=\"row_heading level0 row9\" >2011</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row9_col0\" class=\"data row9 col0\" >-0.068423</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row10\" class=\"row_heading level0 row10\" >2012</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row10_col0\" class=\"data row10 col0\" >-0.023359</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row11\" class=\"row_heading level0 row11\" >2013</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row11_col0\" class=\"data row11 col0\" >-0.007865</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row12\" class=\"row_heading level0 row12\" >2014</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row12_col0\" class=\"data row12 col0\" >0.039288</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row13\" class=\"row_heading level0 row13\" >2015</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row13_col0\" class=\"data row13 col0\" >0.070262</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row14\" class=\"row_heading level0 row14\" >2016</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row14_col0\" class=\"data row14 col0\" >-0.044730</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row15\" class=\"row_heading level0 row15\" >2017</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row15_col0\" class=\"data row15 col0\" >0.049316</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row16\" class=\"row_heading level0 row16\" >2018</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row16_col0\" class=\"data row16 col0\" >-0.030712</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row17\" class=\"row_heading level0 row17\" >2019</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row17_col0\" class=\"data row17 col0\" >-0.015553</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_341fe692_f5a1_11ea_87db_acde48001122level0_row18\" class=\"row_heading level0 row18\" >2020</th>\n",
       "                        <td id=\"T_341fe692_f5a1_11ea_87db_acde48001122row18_col0\" class=\"data row18 col0\" >-0.049961</td>\n",
       "            </tr>\n",
       "    </tbody></table>"
      ],
      "text/plain": [
       "<pandas.io.formats.style.Styler at 0x7fec4dc11610>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th></th>\n",
       "      <th>Summary</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>Annualized Returns</th>\n",
       "      <td>-0.003329</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Annualized Volatility</th>\n",
       "      <td>0.097833</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Sharpe Ratio (assuming $R_f$ = 0)</th>\n",
       "      <td>-0.018946</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                                    Summary\n",
       "Annualized Returns                -0.003329\n",
       "Annualized Volatility              0.097833\n",
       "Sharpe Ratio (assuming $R_f$ = 0) -0.018946"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "====================================================================================================\n"
     ]
    },
    {
     "data": {
      "text/html": [
       "<style  type=\"text/css\" >\n",
       "</style><table id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122\" ><caption>Currencies</caption><thead>    <tr>        <th class=\"blank level0\" ></th>        <th class=\"col_heading level0 col0\" >calendar_year_return</th>    </tr>    <tr>        <th class=\"index_name level0\" >Date</th>        <th class=\"blank\" ></th>    </tr></thead><tbody>\n",
       "                <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row0\" class=\"row_heading level0 row0\" >2002</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row0_col0\" class=\"data row0 col0\" >0.024058</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row1\" class=\"row_heading level0 row1\" >2003</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row1_col0\" class=\"data row1 col0\" >0.094440</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row2\" class=\"row_heading level0 row2\" >2004</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row2_col0\" class=\"data row2 col0\" >-0.013657</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row3\" class=\"row_heading level0 row3\" >2005</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row3_col0\" class=\"data row3 col0\" >-0.036125</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row4\" class=\"row_heading level0 row4\" >2006</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row4_col0\" class=\"data row4 col0\" >-0.001895</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row5\" class=\"row_heading level0 row5\" >2007</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row5_col0\" class=\"data row5 col0\" >0.000725</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row6\" class=\"row_heading level0 row6\" >2008</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row6_col0\" class=\"data row6 col0\" >0.099092</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row7\" class=\"row_heading level0 row7\" >2009</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row7_col0\" class=\"data row7 col0\" >-0.106025</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row8\" class=\"row_heading level0 row8\" >2010</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row8_col0\" class=\"data row8 col0\" >-0.049425</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row9\" class=\"row_heading level0 row9\" >2011</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row9_col0\" class=\"data row9 col0\" >-0.041039</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row10\" class=\"row_heading level0 row10\" >2012</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row10_col0\" class=\"data row10 col0\" >-0.032922</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row11\" class=\"row_heading level0 row11\" >2013</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row11_col0\" class=\"data row11 col0\" >0.090975</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row12\" class=\"row_heading level0 row12\" >2014</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row12_col0\" class=\"data row12 col0\" >-0.011465</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row13\" class=\"row_heading level0 row13\" >2015</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row13_col0\" class=\"data row13 col0\" >-0.009285</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row14\" class=\"row_heading level0 row14\" >2016</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row14_col0\" class=\"data row14 col0\" >0.059619</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row15\" class=\"row_heading level0 row15\" >2017</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row15_col0\" class=\"data row15 col0\" >0.029123</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row16\" class=\"row_heading level0 row16\" >2018</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row16_col0\" class=\"data row16 col0\" >-0.028743</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row17\" class=\"row_heading level0 row17\" >2019</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row17_col0\" class=\"data row17 col0\" >-0.014778</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122level0_row18\" class=\"row_heading level0 row18\" >2020</th>\n",
       "                        <td id=\"T_343b3c8a_f5a1_11ea_87db_acde48001122row18_col0\" class=\"data row18 col0\" >-0.036071</td>\n",
       "            </tr>\n",
       "    </tbody></table>"
      ],
      "text/plain": [
       "<pandas.io.formats.style.Styler at 0x7fec4dc11e10>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th></th>\n",
       "      <th>Summary</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>Annualized Returns</th>\n",
       "      <td>-0.000491</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Annualized Volatility</th>\n",
       "      <td>0.070615</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Sharpe Ratio (assuming $R_f$ = 0)</th>\n",
       "      <td>-0.002142</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                                    Summary\n",
       "Annualized Returns                -0.000491\n",
       "Annualized Volatility              0.070615\n",
       "Sharpe Ratio (assuming $R_f$ = 0) -0.002142"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "====================================================================================================\n"
     ]
    },
    {
     "data": {
      "text/html": [
       "<style  type=\"text/css\" >\n",
       "</style><table id=\"T_3454808c_f5a1_11ea_87db_acde48001122\" ><caption>Commod</caption><thead>    <tr>        <th class=\"blank level0\" ></th>        <th class=\"col_heading level0 col0\" >calendar_year_return</th>    </tr>    <tr>        <th class=\"index_name level0\" >Date</th>        <th class=\"blank\" ></th>    </tr></thead><tbody>\n",
       "                <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row0\" class=\"row_heading level0 row0\" >2002</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row0_col0\" class=\"data row0 col0\" >-0.044172</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row1\" class=\"row_heading level0 row1\" >2003</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row1_col0\" class=\"data row1 col0\" >-0.148021</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row2\" class=\"row_heading level0 row2\" >2004</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row2_col0\" class=\"data row2 col0\" >0.005226</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row3\" class=\"row_heading level0 row3\" >2005</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row3_col0\" class=\"data row3 col0\" >0.168395</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row4\" class=\"row_heading level0 row4\" >2006</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row4_col0\" class=\"data row4 col0\" >0.228971</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row5\" class=\"row_heading level0 row5\" >2007</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row5_col0\" class=\"data row5 col0\" >0.049709</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row6\" class=\"row_heading level0 row6\" >2008</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row6_col0\" class=\"data row6 col0\" >0.356241</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row7\" class=\"row_heading level0 row7\" >2009</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row7_col0\" class=\"data row7 col0\" >-0.290566</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row8\" class=\"row_heading level0 row8\" >2010</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row8_col0\" class=\"data row8 col0\" >0.018869</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row9\" class=\"row_heading level0 row9\" >2011</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row9_col0\" class=\"data row9 col0\" >0.036266</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row10\" class=\"row_heading level0 row10\" >2012</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row10_col0\" class=\"data row10 col0\" >-0.201700</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row11\" class=\"row_heading level0 row11\" >2013</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row11_col0\" class=\"data row11 col0\" >0.165785</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row12\" class=\"row_heading level0 row12\" >2014</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row12_col0\" class=\"data row12 col0\" >0.110457</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row13\" class=\"row_heading level0 row13\" >2015</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row13_col0\" class=\"data row13 col0\" >0.035128</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row14\" class=\"row_heading level0 row14\" >2016</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row14_col0\" class=\"data row14 col0\" >-0.308718</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row15\" class=\"row_heading level0 row15\" >2017</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row15_col0\" class=\"data row15 col0\" >0.019870</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row16\" class=\"row_heading level0 row16\" >2018</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row16_col0\" class=\"data row16 col0\" >-0.112933</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row17\" class=\"row_heading level0 row17\" >2019</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row17_col0\" class=\"data row17 col0\" >-0.099779</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_3454808c_f5a1_11ea_87db_acde48001122level0_row18\" class=\"row_heading level0 row18\" >2020</th>\n",
       "                        <td id=\"T_3454808c_f5a1_11ea_87db_acde48001122row18_col0\" class=\"data row18 col0\" >-0.442822</td>\n",
       "            </tr>\n",
       "    </tbody></table>"
      ],
      "text/plain": [
       "<pandas.io.formats.style.Styler at 0x7fec4dc11d90>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th></th>\n",
       "      <th>Summary</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>Annualized Returns</th>\n",
       "      <td>-0.044557</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Annualized Volatility</th>\n",
       "      <td>0.381381</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Sharpe Ratio (assuming $R_f$ = 0)</th>\n",
       "      <td>-0.053051</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                                    Summary\n",
       "Annualized Returns                -0.044557\n",
       "Annualized Volatility              0.381381\n",
       "Sharpe Ratio (assuming $R_f$ = 0) -0.053051"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "====================================================================================================\n"
     ]
    },
    {
     "data": {
      "text/html": [
       "<style  type=\"text/css\" >\n",
       "</style><table id=\"T_34736574_f5a1_11ea_87db_acde48001122\" ><caption>EQ Factors </caption><thead>    <tr>        <th class=\"blank level0\" ></th>        <th class=\"col_heading level0 col0\" >calendar_year_return</th>    </tr>    <tr>        <th class=\"index_name level0\" >Date</th>        <th class=\"blank\" ></th>    </tr></thead><tbody>\n",
       "                <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row0\" class=\"row_heading level0 row0\" >2002</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row0_col0\" class=\"data row0 col0\" >0.000000</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row1\" class=\"row_heading level0 row1\" >2003</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row1_col0\" class=\"data row1 col0\" >-0.021777</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row2\" class=\"row_heading level0 row2\" >2004</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row2_col0\" class=\"data row2 col0\" >-0.035077</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row3\" class=\"row_heading level0 row3\" >2005</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row3_col0\" class=\"data row3 col0\" >0.023022</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row4\" class=\"row_heading level0 row4\" >2006</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row4_col0\" class=\"data row4 col0\" >-0.033745</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row5\" class=\"row_heading level0 row5\" >2007</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row5_col0\" class=\"data row5 col0\" >0.061733</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row6\" class=\"row_heading level0 row6\" >2008</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row6_col0\" class=\"data row6 col0\" >0.018270</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row7\" class=\"row_heading level0 row7\" >2009</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row7_col0\" class=\"data row7 col0\" >0.016557</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row8\" class=\"row_heading level0 row8\" >2010</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row8_col0\" class=\"data row8 col0\" >-0.011831</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row9\" class=\"row_heading level0 row9\" >2011</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row9_col0\" class=\"data row9 col0\" >-0.175542</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row10\" class=\"row_heading level0 row10\" >2012</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row10_col0\" class=\"data row10 col0\" >0.014983</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row11\" class=\"row_heading level0 row11\" >2013</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row11_col0\" class=\"data row11 col0\" >0.110600</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row12\" class=\"row_heading level0 row12\" >2014</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row12_col0\" class=\"data row12 col0\" >-0.079264</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row13\" class=\"row_heading level0 row13\" >2015</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row13_col0\" class=\"data row13 col0\" >0.058069</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row14\" class=\"row_heading level0 row14\" >2016</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row14_col0\" class=\"data row14 col0\" >-0.072724</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row15\" class=\"row_heading level0 row15\" >2017</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row15_col0\" class=\"data row15 col0\" >0.009679</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row16\" class=\"row_heading level0 row16\" >2018</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row16_col0\" class=\"data row16 col0\" >0.037166</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row17\" class=\"row_heading level0 row17\" >2019</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row17_col0\" class=\"data row17 col0\" >0.026270</td>\n",
       "            </tr>\n",
       "            <tr>\n",
       "                        <th id=\"T_34736574_f5a1_11ea_87db_acde48001122level0_row18\" class=\"row_heading level0 row18\" >2020</th>\n",
       "                        <td id=\"T_34736574_f5a1_11ea_87db_acde48001122row18_col0\" class=\"data row18 col0\" >0.427873</td>\n",
       "            </tr>\n",
       "    </tbody></table>"
      ],
      "text/plain": [
       "<pandas.io.formats.style.Styler at 0x7fec4dc11710>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th></th>\n",
       "      <th>Summary</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>Annualized Returns</th>\n",
       "      <td>0.014129</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Annualized Volatility</th>\n",
       "      <td>0.228858</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>Sharpe Ratio (assuming $R_f$ = 0)</th>\n",
       "      <td>0.028603</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "                                    Summary\n",
       "Annualized Returns                 0.014129\n",
       "Annualized Volatility              0.228858\n",
       "Sharpe Ratio (assuming $R_f$ = 0)  0.028603"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "data": {
      "application/javascript": [
       "\n",
       "            setTimeout(function() {\n",
       "                var nbb_cell_id = 11;\n",
       "                var nbb_unformatted_code = \"for asset_class in data.columns.get_level_values(\\\"asset_class\\\").unique():\\n    print(\\\"=\\\" * 100)\\n    csms = CrossSectionalMomentumStrategy(\\n        asset_class=asset_class, asset_class_history=data\\n    )\\n    csms.execute_backtest()\";\n",
       "                var nbb_formatted_code = \"for asset_class in data.columns.get_level_values(\\\"asset_class\\\").unique():\\n    print(\\\"=\\\" * 100)\\n    csms = CrossSectionalMomentumStrategy(\\n        asset_class=asset_class, asset_class_history=data\\n    )\\n    csms.execute_backtest()\";\n",
       "                var nbb_cells = Jupyter.notebook.get_cells();\n",
       "                for (var i = 0; i < nbb_cells.length; ++i) {\n",
       "                    if (nbb_cells[i].input_prompt_number == nbb_cell_id) {\n",
       "                        if (nbb_cells[i].get_text() == nbb_unformatted_code) {\n",
       "                             nbb_cells[i].set_text(nbb_formatted_code);\n",
       "                        }\n",
       "                        break;\n",
       "                    }\n",
       "                }\n",
       "            }, 500);\n",
       "            "
      ],
      "text/plain": [
       "<IPython.core.display.Javascript object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "time: 1.04 s\n"
     ]
    }
   ],
   "source": [
    "for asset_class in data.columns.get_level_values(\"asset_class\").unique():\n",
    "    print(\"=\" * 100)\n",
    "    csms = CrossSectionalMomentumStrategy(\n",
    "        asset_class=asset_class, asset_class_history=data\n",
    "    )\n",
    "    csms.execute_backtest()"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.8.5"
  },
  "latex_envs": {
   "LaTeX_envs_menu_present": true,
   "autoclose": false,
   "autocomplete": true,
   "bibliofile": "biblio.bib",
   "cite_by": "apalike",
   "current_citInitial": 1,
   "eqLabelWithNumbers": true,
   "eqNumInitial": 1,
   "hotkeys": {
    "equation": "Ctrl-E",
    "itemize": "Ctrl-I"
   },
   "labels_anchors": false,
   "latex_user_defs": false,
   "report_style_numbering": false,
   "user_envs_cfg": false
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
